module alt_fifo_block #
	(
		parameter DATA_WIDTH = 8,
		parameter SIZE = 3
	)
	(
		input clock,
		input reset, 
		input read,
		input write,
		input write_next,
		input [DATA_WIDTH-1:0] data_in,
		output [DATA_WIDTH-1:0] data_out,
		output reg valid,
		output is_space
	);
	
	//ceil of the log base 2
	function integer alt_clogb2;
	input [31:0] value;
	integer i;
	begin
		alt_clogb2 = 32;
		for (i=31; i>0; i=i-1) begin
			if (2**i>=value) begin
				alt_clogb2 = i;
			end
		end
	end
	endfunction
	
	localparam COUNT_W = alt_clogb2(SIZE);
	wire [DATA_WIDTH-1:0] data_next;
	reg [DATA_WIDTH-1:0] data_curr;
	reg [COUNT_W-1:0] count, r_addr, w_addr;
	wire [COUNT_W-1:0] r_addr_next, w_addr_next; 
	wire do_update, stop;
	reg read_str;
	
	assign do_update = (w_addr == r_addr) && write;
   assign stop = (count == 0) && (r_addr != w_addr);
	
	alt_pass_through_ram #
	(
		.SIZE (SIZE),
		.WIDTH (DATA_WIDTH),
		.ADDR_WIDTH (COUNT_W)
	) ram_inst (
		.q (data_next),
		.d (data_in),
		.write_address (w_addr),
		.read_address (r_addr_next),
		.we (write),
		.clk (clock)
	);
 
	always @ (posedge clock or posedge reset) begin
		if (reset) begin
			count <= 0;
			r_addr <= 0;
			w_addr <= 0;
			valid <= 1'b0;
			read_str <= 1'b0;
		end else begin
		   read_str <= read && valid;
			if (count > 0) begin
			   count <= count + write - read;
			end else begin
			   count <= count + write;
			end
			if ((read) && valid) begin
				r_addr <= r_addr_next;
			end
			if (write) begin
				w_addr <= w_addr_next;
			end
			if (read || write) begin
			    valid <= write || (count > 1);
			end
			if (do_update) begin
			    data_curr <= data_in;
			end else begin
            if (read_str) begin
               data_curr <= data_next;
            end
         end
		end
	end

	assign r_addr_next = (r_addr < SIZE-1) ? r_addr+1 : 0;
	assign w_addr_next = (w_addr < SIZE-1) ? w_addr+1 : 0;
	assign is_space = ((count < (SIZE-2)) || ((count < (SIZE-1)) && !(write && write_next)) || ((count < SIZE) && !(write || write_next)));
	assign data_out = (read_str) ? data_next : data_curr;

endmodule 