module alt_shift_block #
	(
		parameter DATA_WIDTH = 8,
		parameter SHIFT_LENGTH = 3
	)
	(
		input clock,
		input enable,
		input [DATA_WIDTH-1:0] data_in,
		output [DATA_WIDTH-1:0] data_out
	);
	
	generate
	   if (SHIFT_LENGTH > 1) begin
	      reg [DATA_WIDTH-1:0] sr [SHIFT_LENGTH-1:0];
	      integer n;
	      always @ (posedge clock)
	      begin
		      if (enable == 1'b1) begin
			      for (n=SHIFT_LENGTH-1; n>0; n=n-1) begin
				      sr[n] <= sr[n-1];
			      end
			      sr[0] <= data_in;
		      end
	      end
	      assign data_out = sr[SHIFT_LENGTH-1];
	   end else begin
	      reg [DATA_WIDTH-1:0] sr;
	      always @ (posedge clock)
	      begin
		      if (enable == 1'b1) begin
			      sr <= data_in;
		      end
	      end
	      assign data_out = sr;
	   end
	endgenerate
	

endmodule 