module alt_vip_chroma_key

	(	clock,
		reset,
		// Avalon-ST sink interface
		din_0_ready,
		din_0_valid,
		din_0_startofpacket,
		din_0_endofpacket,
		din_0_data, 
		// Avalon-ST source interfaces
		//video_data
		dout_0_ready,
		dout_0_valid,
		dout_0_startofpacket,
		dout_0_endofpacket,
		dout_0_data,
		//alpha_data
		dout_1_ready,
		dout_1_valid,
		dout_1_startofpacket,
		dout_1_endofpacket,
		dout_1_data,
		//Avalon MM Slave interface
		av_chipselect,
		av_readdata,
		av_write,
		av_writedata,
		av_address
	); 
	
	//ceil of the log base 2
	function integer alt_clogb2;
	input [31:0] value;
	integer i;
	begin
		alt_clogb2 = 32;
		for (i=31; i>0; i=i-1) begin
			if (2**i>=value) begin
				alt_clogb2 = i;
			end
		end
	end
	endfunction
	
	parameter BITS_PER_SYMBOL_VIDEO = 8;
	parameter SYMBOLS_PER_BEAT_VIDEO = 3;
	parameter SELECT_422 = 0;
	parameter BITS_PER_SYMBOL_ALPHA = 8;
	parameter NUMBER_REFERENCE_PIXELS = 7;                
	parameter DEFAULT_ALPHA_VALUE = 3;
	parameter FIFO_LEN = 3;                        // should be >= 3	
	parameter DISABLE_VIDEO_REPLACE = 0;
	parameter DISABLE_ALPHA_REPLACE = 0;
	parameter ENABLE_CLEAN_SWITCHING = 1;
	parameter ENABLE_LUMINANCE = 2; 
	parameter ALPHA_SPLITTING = 0;
	parameter ENABLE_RANGE = 0;
	
	//local parameters
	localparam AV_DATA_WIDTH = 32;
	localparam ADDRESS_WIDTH = alt_clogb2(NUMBER_REFERENCE_PIXELS);
	localparam IS_422 = (SELECT_422 > 0) ? 1 : 0;
	localparam FIFO_LEN_INT = (FIFO_LEN > 2) ? FIFO_LEN : 3;
	localparam ADDRESS_WIDTH_2 = ((DISABLE_VIDEO_REPLACE == 0) || (DISABLE_ALPHA_REPLACE == 0)) ? (ADDRESS_WIDTH > 3) ? ADDRESS_WIDTH+2 : 5 : 2;
	localparam SYMBOLS_PER_BEAT_VIDEO_OUT = (((IS_422 > 0) && (SYMBOLS_PER_BEAT_VIDEO > 2)) || (SYMBOLS_PER_BEAT_VIDEO > 3)) ? SYMBOLS_PER_BEAT_VIDEO-1 : SYMBOLS_PER_BEAT_VIDEO;
	
	input		wire clock;
	input		wire reset;
	// Avalon-ST sink interface
	output	wire din_0_ready;
	input	wire din_0_valid;
	input	wire din_0_startofpacket;
	input	wire din_0_endofpacket;
	input	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] din_0_data; 
	// Avalon-ST source interfaces
	//video_data
	input	 wire dout_0_ready;
	output	wire dout_0_valid;
	output	wire dout_0_startofpacket;
	output wire dout_0_endofpacket;
	output	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO_OUT - 1:0] dout_0_data;
	//alpha_data
	input	 wire dout_1_ready;
	output	wire dout_1_valid;
	output	wire dout_1_startofpacket;
	output	wire dout_1_endofpacket;
	output	wire [BITS_PER_SYMBOL_ALPHA - 1:0] dout_1_data;
		
	//Avalon MM Slave interface
	input   wire                              av_chipselect;
	output  wire [AV_DATA_WIDTH-1:0]          av_readdata;
	input   wire                              av_write;
	input   wire [AV_DATA_WIDTH-1:0]          av_writedata;
	input   wire [ADDRESS_WIDTH_2-1:0]		  av_address;
	
	wire video_ready, video_valid, video_sop, video_eop; 
	wire alpha_ready, alpha_valid, alpha_sop, alpha_eop;
	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO_OUT-1:0] video_data;
	wire [BITS_PER_SYMBOL_ALPHA-1:0] alpha_data;
	wire enable_0_int, enable_1_int, synced_0_int, synced_1_int;

	assign enable_0_int = 1'b1;
	assign enable_1_int = 1'b1;

	generate 
		if (ALPHA_SPLITTING > 0) begin
			if ((DISABLE_VIDEO_REPLACE == 0) || (DISABLE_ALPHA_REPLACE == 0)) begin
			
				alt_vip_chroma_key_splitting_core #(
					.BITS_PER_SYMBOL_VIDEO (BITS_PER_SYMBOL_VIDEO),
					.SYMBOLS_PER_BEAT_VIDEO_IN (SYMBOLS_PER_BEAT_VIDEO),
					.IS_422 (IS_422),
					.BITS_PER_SYMBOL_ALPHA (BITS_PER_SYMBOL_ALPHA),
					.NUM_COLOURS (NUMBER_REFERENCE_PIXELS),                
					.FIFO_SIZE (FIFO_LEN_INT),
					.DISABLE_VIDEO_REPLACE (DISABLE_VIDEO_REPLACE), 
					.DISABLE_ALPHA_REPLACE (DISABLE_ALPHA_REPLACE),
					.ENABLE_CLEAN_SWITCHING (ENABLE_CLEAN_SWITCHING),
					.ENABLE_COMPONENT_1 (ENABLE_LUMINANCE),
					.ENABLE_RANGE (ENABLE_RANGE)
				) algorithm (	
					.clock (clock),
					.reset (reset),
					.din_0_ready (din_0_ready),
					.din_0_valid (din_0_valid),
					.din_0_startofpacket (din_0_startofpacket),
					.din_0_endofpacket (din_0_endofpacket),
					.din_0_data (din_0_data), 
					.dout_0_ready (video_ready),
					.dout_0_valid (video_valid),
					.dout_0_startofpacket (video_sop),
					.dout_0_endofpacket (video_eop),
					.dout_0_data (video_data),
					.dout_1_ready (alpha_ready),
					.dout_1_valid (alpha_valid),
					.dout_1_startofpacket (alpha_sop),
					.dout_1_endofpacket (alpha_eop),
					.dout_1_data (alpha_data),
					.av_chipselect (av_chipselect),
					.av_readdata (av_readdata),
					.av_write (av_write),
					.av_writedata (av_writedata),
					.av_address (av_address)
				);	
				
			end else begin
				
				alt_vip_splitting_alpha_source_core #(
					.BITS_PER_SYMBOL_VIDEO (BITS_PER_SYMBOL_VIDEO),
					.SYMBOLS_PER_BEAT_VIDEO_IN (SYMBOLS_PER_BEAT_VIDEO),
					.IS_422 (IS_422),
					.BITS_PER_SYMBOL_ALPHA (BITS_PER_SYMBOL_ALPHA),                
					.FIFO_SIZE (FIFO_LEN_INT)
				) algorithm (	
					.clock (clock),
					.reset (reset),
					.din_0_ready (din_0_ready),
					.din_0_valid (din_0_valid),
					.din_0_startofpacket (din_0_startofpacket),
					.din_0_endofpacket (din_0_endofpacket),
					.din_0_data (din_0_data), 
					.dout_0_ready (video_ready),
					.dout_0_valid (video_valid),
					.dout_0_startofpacket (video_sop),
					.dout_0_endofpacket (video_eop),
					.dout_0_data (video_data),
					.dout_1_ready (alpha_ready),
					.dout_1_valid (alpha_valid),
					.dout_1_startofpacket (alpha_sop),
					.dout_1_endofpacket (alpha_eop),
					.dout_1_data (alpha_data),
					.av_chipselect (av_chipselect),
					.av_readdata (av_readdata),
					.av_write (av_write),
					.av_writedata (av_writedata),
					.av_address (av_address)
				);
				
			end
		end else begin
			if ((DISABLE_VIDEO_REPLACE == 0) || (DISABLE_ALPHA_REPLACE == 0)) begin
				
				alt_vip_chroma_key_core #(
					.BITS_PER_SYMBOL_VIDEO (BITS_PER_SYMBOL_VIDEO),
					.SYMBOLS_PER_BEAT_VIDEO (SYMBOLS_PER_BEAT_VIDEO),
					.IS_422 (IS_422),
					.BITS_PER_SYMBOL_ALPHA (BITS_PER_SYMBOL_ALPHA),
					.NUM_COLOURS (NUMBER_REFERENCE_PIXELS),                
					.DEFAULT_ALPHA_VALUE (DEFAULT_ALPHA_VALUE),
					.FIFO_SIZE (FIFO_LEN_INT),
					.DISABLE_VIDEO_REPLACE (DISABLE_VIDEO_REPLACE), 
					.DISABLE_ALPHA_REPLACE (DISABLE_ALPHA_REPLACE),
					.ENABLE_CLEAN_SWITCHING (ENABLE_CLEAN_SWITCHING),
					.ENABLE_COMPONENT_1 (ENABLE_LUMINANCE),
					.ENABLE_RANGE (ENABLE_RANGE)
				) algorithm (	
					.clock (clock),
					.reset (reset),
					.din_0_ready (din_0_ready),
					.din_0_valid (din_0_valid),
					.din_0_startofpacket (din_0_startofpacket),
					.din_0_endofpacket (din_0_endofpacket),
					.din_0_data (din_0_data), 
					.dout_0_ready (video_ready),
					.dout_0_valid (video_valid),
					.dout_0_startofpacket (video_sop),
					.dout_0_endofpacket (video_eop),
					.dout_0_data (video_data),
					.dout_1_ready (alpha_ready),
					.dout_1_valid (alpha_valid),
					.dout_1_startofpacket (alpha_sop),
					.dout_1_endofpacket (alpha_eop),
					.dout_1_data (alpha_data),
					.av_chipselect (av_chipselect),
					.av_readdata (av_readdata),
					.av_write (av_write),
					.av_writedata (av_writedata),
					.av_address (av_address)
				);
				
			end else begin
    
				alt_vip_alpha_source_core #(
					.BITS_PER_SYMBOL_VIDEO 			(BITS_PER_SYMBOL_VIDEO),
					.SYMBOLS_PER_BEAT_VIDEO 		(SYMBOLS_PER_BEAT_VIDEO),
					.IS_422               		(IS_422),
					.BITS_PER_SYMBOL_ALPHA 			(BITS_PER_SYMBOL_ALPHA),
               .FIFO_SIZE              (FIFO_LEN_INT),
					.DEFAULT_ALPHA_VALUE 			(DEFAULT_ALPHA_VALUE),
					.ENABLE_CLEAN_SWITCHING         (ENABLE_CLEAN_SWITCHING)
				) algorithm (	
					.clock							(clock),
					.reset							(reset),
					.din_0_ready					(din_0_ready),
					.din_0_valid					(din_0_valid),
					.din_0_startofpacket			(din_0_startofpacket),
					.din_0_endofpacket				(din_0_endofpacket),
					.din_0_data						(din_0_data),
					.dout_0_ready					(video_ready),
					.dout_0_valid					(video_valid),
					.dout_0_startofpacket			(video_sop),
					.dout_0_endofpacket				(video_eop),
					.dout_0_data					(video_data),
					.dout_1_ready					(alpha_ready),
					.dout_1_valid					(alpha_valid),
					.dout_1_startofpacket			(alpha_sop),
					.dout_1_endofpacket				(alpha_eop),
					.dout_1_data					(alpha_data),
					.av_chipselect					(av_chipselect),
					.av_readdata					(av_readdata),
					.av_write						(av_write),
					.av_writedata					(av_writedata),
					.av_address						(av_address)
				);
			
			end
		end
	endgenerate
			
	
	alt_vip_common_stream_output #(
		.DATA_WIDTH (BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO_OUT)
	) output_stream_0 (
		.rst (reset),
    	.clk (clock),
    	.dout_ready (dout_0_ready),
    	.dout_valid (dout_0_valid),
    	.dout_data (dout_0_data),
    	.dout_sop (dout_0_startofpacket),
    	.dout_eop (dout_0_endofpacket),
    	.int_ready (video_ready),
    	.int_valid (video_valid),
    	.int_data (video_data),
    	.int_sop (video_sop),
    	.int_eop (video_eop),
    	.enable (enable_0_int),
    	.synced (synced_0_int)
    );
    
    alt_vip_common_stream_output #(
		.DATA_WIDTH (BITS_PER_SYMBOL_ALPHA)
	) output_stream_1 (
		.rst (reset),
    	.clk (clock),
    	.dout_ready (dout_1_ready),
    	.dout_valid (dout_1_valid),
    	.dout_data (dout_1_data),
    	.dout_sop (dout_1_startofpacket),
    	.dout_eop (dout_1_endofpacket),
    	.int_ready (alpha_ready),
    	.int_valid (alpha_valid),
    	.int_data (alpha_data),
    	.int_sop (alpha_sop),
    	.int_eop (alpha_eop),
    	.enable (enable_1_int),
    	.synced (synced_1_int)
    );
	
endmodule
					
			