module alt_vip_chroma_key_splitting_core

	(	clock,
		reset,
		// Avalon-ST sink interface
		din_0_ready,
		din_0_valid,
		din_0_startofpacket,
		din_0_endofpacket,
		din_0_data, 
		// Avalon-ST source interfaces
		//video_data
		dout_0_ready,
		dout_0_valid,
		dout_0_startofpacket,
		dout_0_endofpacket,
		dout_0_data,
		//alpha_data
		dout_1_ready,
		dout_1_valid,
		dout_1_startofpacket,
		dout_1_endofpacket,
		dout_1_data,
		//Avalon MM Slave interface
		av_chipselect,
		av_readdata,
		av_write,
		av_writedata,
		av_address
	); 
	
	//ceil of the log base 2
	function integer alt_clogb2;
	input [31:0] value;
	integer i;
	begin
		alt_clogb2 = 32;
		for (i=31; i>0; i=i-1) begin
			if (2**i>=value) begin
				alt_clogb2 = i;
			end
		end
	end
	endfunction
	
	function integer get_sym_len;
	input integer is_422;
	input integer s_p_b;
	begin
		if (is_422 > 0) begin
			case (s_p_b)
				default	:	get_sym_len = 2;
				2		:	get_sym_len = 4;
				1		:	get_sym_len = 6;
			endcase
		end else begin
			case (s_p_b)
				default	:	get_sym_len = 1;
				3		:	get_sym_len = 2;
				2		:	get_sym_len = 2;
				1		:	get_sym_len = 4;
			endcase
		end
	end
	endfunction
	
	function integer get_alpha_len;
	input integer is_422;
	input integer s_p_b;
	begin
		if (is_422 > 0) begin
			if (s_p_b > 1) begin
				get_alpha_len = 4;
			end else begin
				get_alpha_len = 5;
			end
		end else begin
			get_alpha_len = 3;
		end
	end
	endfunction
	
	function integer next_power_2;
	input integer value;
	integer temp;
	begin
		temp = 1;
		while (temp < value) begin
			temp = temp << 1;
		end
		next_power_2 = temp;
	end
	endfunction
	
	parameter BITS_PER_SYMBOL_VIDEO = 8;
	parameter SYMBOLS_PER_BEAT_VIDEO_IN = 3;
	parameter IS_422 = 0;
	parameter BITS_PER_SYMBOL_ALPHA = 8;
	parameter NUM_COLOURS = 32;                
	parameter FIFO_SIZE = 3;                        // must be >= 3	
	parameter DISABLE_VIDEO_REPLACE = 0;
	parameter DISABLE_ALPHA_REPLACE = 0;
	parameter ENABLE_CLEAN_SWITCHING = 0;
	parameter ENABLE_COMPONENT_1 = 0;              //0 : Disable, 1 : Permanent Enable, 2 : Run time control
	parameter ENABLE_RANGE = 0;
	//local parameters
	localparam SYMBOLS_PER_BEAT_VIDEO = (((IS_422 > 0) && (SYMBOLS_PER_BEAT_VIDEO_IN > 2)) || (SYMBOLS_PER_BEAT_VIDEO_IN > 3)) ? SYMBOLS_PER_BEAT_VIDEO_IN-1 : SYMBOLS_PER_BEAT_VIDEO_IN;
	localparam AV_DATA_WIDTH = 32;
	localparam AV_STR_WIDTH = (BITS_PER_SYMBOL_ALPHA > BITS_PER_SYMBOL_VIDEO) ? BITS_PER_SYMBOL_ALPHA : BITS_PER_SYMBOL_VIDEO;
	localparam ADDRESS_WIDTH = alt_clogb2(NUM_COLOURS);
	localparam ADDRESS_WIDTH_LOCAL = (ENABLE_CLEAN_SWITCHING > 0) ? ADDRESS_WIDTH+1 : ADDRESS_WIDTH;
	localparam SHIFT_LENGTH = (4 - SYMBOLS_PER_BEAT_VIDEO - IS_422)*(IS_422+1);  
	localparam ADDRESS_WIDTH_2 = (ADDRESS_WIDTH > 3) ? 5 : ADDRESS_WIDTH+2;
	localparam STR_SYMBOLS = (ENABLE_COMPONENT_1 == 0) ? 2 : (IS_422) ? 4 : 3;
	localparam EOP_POS = (IS_422) ? (SYMBOLS_PER_BEAT_VIDEO_IN == 1) ? 5 : 4 : 3;
	localparam LOCAL_MEM_DATA_W = (DISABLE_ALPHA_REPLACE) ? STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO : (DISABLE_VIDEO_REPLACE) ? BITS_PER_SYMBOL_ALPHA : STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO + BITS_PER_SYMBOL_ALPHA;
	localparam MEM_SIZE = (ENABLE_CLEAN_SWITCHING > 0) ? 2*next_power_2(NUM_COLOURS) : NUM_COLOURS;
	localparam SYM_COUNT_LEN = get_sym_len(IS_422, SYMBOLS_PER_BEAT_VIDEO_IN);
	localparam ALPHA_SHIFT_LEN = get_alpha_len(IS_422, SYMBOLS_PER_BEAT_VIDEO_IN);
	localparam LAST_INDEX = ((IS_422 > 0) && (SYMBOLS_PER_BEAT_VIDEO_IN < 3)) ? SYM_COUNT_LEN-2 : SHIFT_LENGTH-1;
	localparam RANGE_X2 = (ENABLE_RANGE > 0) ? 2 : 1;
	
	
	input		wire clock;
	input		wire reset;
	// Avalon-ST sink interface
	output	wire din_0_ready;
	input	wire din_0_valid;
	input	wire din_0_startofpacket;
	input	wire din_0_endofpacket;
	input	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO_IN - 1:0] din_0_data; 
	// Avalon-ST source interfaces
	//video_data
	input	wire dout_0_ready;
	output	wire dout_0_valid;
	output	wire dout_0_startofpacket;
	output	wire dout_0_endofpacket;
	output	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] dout_0_data;
	//alpha_data
	input	wire dout_1_ready;
	output	wire dout_1_valid;
	output	wire dout_1_startofpacket;
	output	wire dout_1_endofpacket;
	output	wire [BITS_PER_SYMBOL_ALPHA - 1:0] dout_1_data;
		
	//Avalon MM Slave interface
	input   wire                              av_chipselect;
	output  reg  [AV_DATA_WIDTH-1:0]          av_readdata;
	input   wire                              av_write;
	input   wire [AV_DATA_WIDTH-1:0]          av_writedata;
	input   wire [ADDRESS_WIDTH_2-1:0]		  av_address;
	
	function integer alt_power_2;
	input [31:0] power;
	integer m;
	begin
		m=0;
		alt_power_2 = 1;
		while (m<power) begin
			alt_power_2 = alt_power_2*2;
			m=m+1;
		end
	end
	endfunction
	
	function integer alt_repack;
	input [31:0] bit_num;
	input [NUM_COLOURS - 1:0] select_sig;
	integer out_word, max;
	integer j, k, l;
	begin
		l=0;
		out_word = 0;
		for (j=alt_power_2(bit_num); j < NUM_COLOURS; j=j+alt_power_2(bit_num+1)) begin
		   max = NUM_COLOURS;
		   if ((j + alt_power_2(bit_num)) < max) begin
		       max = j + alt_power_2(bit_num);
		   end
			for (k=j; k < max; k=k+1) begin
				out_word[l] = select_sig[k];
				l=l+1;
			end
		end
		alt_repack = out_word;
	end
	endfunction
	
	reg [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] data_str [SHIFT_LENGTH+2:0];
	reg [NUM_COLOURS-1:0] select;
	reg master_enable, running, do_alpha_replace, do_video_replace;
	reg [SHIFT_LENGTH+2:0] eop_str, sop_str, valid_str;
	reg no_replace;
	reg ready_reg;
	reg [SHIFT_LENGTH+2:0] is_video;
	reg av_chipselect_str, av_write_str;
	reg [AV_STR_WIDTH-1:0] av_writedata_str, av_writedata_str_1, av_writedata_str_2, av_writedata_str_3, av_writedata_str_4, av_writedata_str_5, av_writedata_str_6, av_writedata_str_7;
	reg [ADDRESS_WIDTH_2-1:0] av_address_str;
	reg [SYM_COUNT_LEN-1:0] sym_count;
	reg [SHIFT_LENGTH+1:0] sym_delayed, do_alpha_sop, early_eop;
	reg [1:0] sym_delayed_1;
	reg [SHIFT_LENGTH+2:0] flush;
	reg [RANGE_X2*STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO - 1:0] match_value [NUM_COLOURS-1:0];
	reg [LOCAL_MEM_DATA_W - 1:0] mem_data_str;
	reg sink_ready, do_switch;
	reg video_in_ready;
	reg comp_1_1, alpha_replace_1, video_replace_1;
	reg [NUM_COLOURS-1:0] col_enables_1;
	
	integer init_i, r, u, init_j;
	
	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO+1:0] video_fifo_data_in, video_fifo_data_out;
	wire [BITS_PER_SYMBOL_ALPHA+1:0] alpha_fifo_data_in, alpha_fifo_data_out;
	wire [ADDRESS_WIDTH_LOCAL-1:0] r_address, w_address;
	wire [ADDRESS_WIDTH-1:0] address;
	wire alpha_valid_in, video_valid_in, write_en, alpha_valid_next, video_valid_next;
	wire [RANGE_X2*BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] comp_in [NUM_COLOURS-1:0];
	wire alpha_ready, video_ready, video_fifo_valid_out, alpha_fifo_valid_out;
	wire [LOCAL_MEM_DATA_W - 1:0] mem_data_out, mem_data_in;
	wire [RANGE_X2*STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO - 1:0] match_in;
	wire video_in_valid, video_in_sop, video_in_eop;
	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] video_in_data;
	wire enable_sig_video, enable_sig_alpha;
	wire [BITS_PER_SYMBOL_ALPHA-1:0] alpha_fifo_in;
	wire allow_flush;
	wire [BITS_PER_SYMBOL_VIDEO-1:0] alpha_str_dummy;
	wire [BITS_PER_SYMBOL_ALPHA-1:0] alpha_save_in;
	wire second_alpha_sig;
	wire comp_1, alpha_replace, video_replace;
	wire [NUM_COLOURS-1:0] col_enables;
	wire comp_1_shift, comp_1_disable;
	
	//stores alpha values and replacement pixel values
	alt_mem_block #(
		.DATA_WIDTH (LOCAL_MEM_DATA_W),
		.ADDR_W (ADDRESS_WIDTH_LOCAL),
		.MEM_DEPTH (MEM_SIZE)
	) mem_inst (
		.clock  (clock),
		.r_address (r_address),
		.w_address (w_address),
		.data_in (mem_data_in),
		.data_out (mem_data_out),
		.write_en (write_en),
		.read_en (enable_sig_video && ready_reg)
	);
	
	alt_fifo_block #(
		.DATA_WIDTH (BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO + 2),
		.SIZE (FIFO_SIZE)
	) fifo_inst_0 (
		.clock  (clock),
		.reset (reset),
		.read (dout_0_ready),
		.write (video_valid_in),
		.write_next (video_valid_next),
		.data_in (video_fifo_data_in),
		.data_out (video_fifo_data_out),
		.valid (video_fifo_valid_out),
		.is_space (video_ready)
	);
	
	alt_fifo_block #(
		.DATA_WIDTH (BITS_PER_SYMBOL_ALPHA + 2),
		.SIZE (FIFO_SIZE)
	) fifo_inst_1 (
		.clock  (clock),
		.reset (reset),
		.read (dout_1_ready),
		.write (alpha_valid_in),
		.write_next (alpha_valid_next),
		.data_in (alpha_fifo_data_in),
		.data_out (alpha_fifo_data_out),
		.valid (alpha_fifo_valid_out),
		.is_space (alpha_ready)
	);
	
	generate
		if ((SYMBOLS_PER_BEAT_VIDEO_IN == 4) || ((SYMBOLS_PER_BEAT_VIDEO_IN == 3) && (IS_422 > 0))) begin
			
			wire [BITS_PER_SYMBOL_VIDEO-1:0] alpha_in_data;
			reg [BITS_PER_SYMBOL_VIDEO-1:0] alpha_str_0;
			
			alt_vip_control_packet_rearrange #(
				.BITS_PER_SYMBOL (BITS_PER_SYMBOL_VIDEO),
				.SYMBOLS_PER_BEAT_IN  (SYMBOLS_PER_BEAT_VIDEO_IN)
			) rearrange (	
				.clock  (clock),
				.reset  (reset),
				.din_0_ready  (din_0_ready),
				.din_0_valid  (din_0_valid),
				.din_0_startofpacket  (din_0_startofpacket),
				.din_0_endofpacket  (din_0_endofpacket),
				.din_0_data  (din_0_data), 
				.dout_0_ready  (video_in_ready),
				.dout_0_valid  (video_in_valid),
				.dout_0_startofpacket  (video_in_sop),
				.dout_0_endofpacket  (video_in_eop),
				.dout_0_data  (video_in_data),
				.dout_0_split_symbol  (alpha_in_data)
			);
			
			always@(posedge clock) begin
		      if (ready_reg) begin
			      alpha_str_0 <= alpha_in_data;
				end
			end
			
			assign alpha_str_dummy = alpha_str_0;
		
		end else begin
			assign alpha_str_dummy = data_str[0][BITS_PER_SYMBOL_ALPHA-1:0];
			assign din_0_ready = video_in_ready;
			assign video_in_valid = din_0_valid;
			assign video_in_sop = din_0_startofpacket;
			assign video_in_eop = din_0_endofpacket;
			assign video_in_data = din_0_data;
			
		end
	endgenerate
	
	generate
		if (IS_422 > 0) begin
			if (SYMBOLS_PER_BEAT_VIDEO_IN == 2) begin
				assign enable_sig_video = valid_str[0] && !(sym_count[3] || sym_count[1]);
				assign enable_sig_alpha = valid_str[0] && (sym_count[3] || sym_count[1]);
			end else begin
			   if (SYMBOLS_PER_BEAT_VIDEO_IN == 1) begin
				   assign enable_sig_video = valid_str[0] && !(sym_count[2] || sym_count[5]);
				   assign enable_sig_alpha = valid_str[0] && (sym_count[2] || sym_count[5]);
			   end else begin
				   assign enable_sig_video = valid_str[0];
				   assign enable_sig_alpha = valid_str[0];
				end
			end
			if (SYMBOLS_PER_BEAT_VIDEO_IN == 1) begin
				reg [1:0] second_alpha;
				always@(posedge clock or posedge reset) begin
					if (reset) begin
						second_alpha <= 0;
					end else begin
						if (ready_reg && (enable_sig_video || (flush[SHIFT_LENGTH+2] && allow_flush))) begin
							second_alpha[1] <= second_alpha[0];
						end
						if (ready_reg && (enable_sig_video ||(flush[SHIFT_LENGTH+1] && allow_flush))) begin
							second_alpha[0] <= ready_reg && (enable_sig_video || (flush[SHIFT_LENGTH+2] && allow_flush)) && valid_str[SHIFT_LENGTH+2] && (is_video[SHIFT_LENGTH+2] && sym_delayed[SHIFT_LENGTH+1]);
						end
					end
				end
				assign second_alpha_sig = second_alpha[1];
			end else begin
				reg second_alpha;
				always@(posedge clock or posedge reset) begin
					if (reset) begin
						second_alpha <= 1'b0;
					end else begin
						if (ready_reg && (enable_sig_video ||(flush[SHIFT_LENGTH+2] && allow_flush))) begin
							second_alpha <= ready_reg && (enable_sig_video || (flush[SHIFT_LENGTH+2] && allow_flush)) && valid_str[SHIFT_LENGTH+2] && (is_video[SHIFT_LENGTH+2] && sym_delayed[SHIFT_LENGTH+1]);//sym_delayed[SHIFT_LENGTH+1];
						end
					end
				end
				assign second_alpha_sig = second_alpha;
			end
		end else begin
			if ((SYMBOLS_PER_BEAT_VIDEO_IN == 1) || (SYMBOLS_PER_BEAT_VIDEO_IN == 3)) begin
				assign enable_sig_video = valid_str[0] && !(sym_count[SYM_COUNT_LEN-1]);
				assign enable_sig_alpha = valid_str[0] && sym_count[SYM_COUNT_LEN-1];
			end else begin
				assign enable_sig_video = valid_str[0];
				assign enable_sig_alpha = valid_str[0];
			end
		end
		if ((IS_422 > 0) || (SYMBOLS_PER_BEAT_VIDEO_IN != 2)) begin
			reg [BITS_PER_SYMBOL_ALPHA-1:0] alpha_str [ALPHA_SHIFT_LEN-1:0];
			integer v;
			assign alpha_fifo_in = alpha_str[ALPHA_SHIFT_LEN-1];
			always@(posedge clock) begin
				if (ready_reg && (enable_sig_alpha ||flush[1])) begin
					alpha_str[0] <= alpha_str_dummy;
				end
			end
			if (ALPHA_SHIFT_LEN > 1) begin
				if (IS_422 > 0) begin
					always@(posedge clock) begin
						if (ready_reg && (enable_sig_video ||(flush[3] && allow_flush))) begin
							alpha_str[2] <= alpha_str[1];
						end
						if (ready_reg && (enable_sig_video ||(flush[2] && allow_flush))) begin
							alpha_str[1] <= alpha_str[0];
						end
						if (ready_reg && (enable_sig_video ||(flush[4] && allow_flush))) begin
							if (do_alpha_replace) begin
								alpha_str[3] <= alpha_save_in;
							end else begin
								alpha_str[3] <= alpha_str[2];
							end
						end
					end
					if  (SYMBOLS_PER_BEAT_VIDEO_IN == 1) begin
						always@(posedge clock) begin
							if (ready_reg && (enable_sig_video ||(flush[5] && allow_flush))) begin
								alpha_str[4] <= alpha_str[3];
							end
						end
					end
				end else begin
					always@(posedge clock) begin
						for (v = ALPHA_SHIFT_LEN-1; v > 0; v=v-1) begin
							if (ready_reg && (enable_sig_video ||(flush[v+1] && allow_flush))) begin
								alpha_str[v] <= alpha_str[v-1];
							end
						end
					end
				end
			end
		end else begin
			assign alpha_fifo_in = data_str[3][BITS_PER_SYMBOL_VIDEO+BITS_PER_SYMBOL_ALPHA-1:BITS_PER_SYMBOL_VIDEO];
		end
	endgenerate
	
	assign write_en = av_write_str && (av_address_str[ADDRESS_WIDTH_2-1:ADDRESS_WIDTH_2-2] == 2'b01);
	//input data to the output shift registers
	assign video_fifo_data_in[1:0] = {sop_str[SHIFT_LENGTH+2],eop_str[SHIFT_LENGTH+2]};
	assign alpha_fifo_data_in[1:0] = {do_alpha_sop[SHIFT_LENGTH+1],(eop_str[EOP_POS] || early_eop[SHIFT_LENGTH+1])};
	//output shift register enables
	//valid inputs to output shift registers 
	assign video_valid_in = ready_reg && (enable_sig_video || (flush[SHIFT_LENGTH+2] && allow_flush)) && valid_str[SHIFT_LENGTH+2];
	assign video_valid_next = video_in_ready && (valid_str[SHIFT_LENGTH+2] || valid_str[SHIFT_LENGTH+1]);
	assign alpha_valid_next = video_in_ready && (valid_str[SHIFT_LENGTH+1] || valid_str[SHIFT_LENGTH+2]);
	//assinging outputs to shift register outputs 
	assign dout_0_valid = video_fifo_valid_out;
	assign dout_0_startofpacket = video_fifo_data_out[1];
	assign dout_0_endofpacket = video_fifo_data_out[0];
	assign dout_0_data = video_fifo_data_out[BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO+1:2];
	assign dout_1_valid = alpha_fifo_valid_out;
	assign dout_1_startofpacket = alpha_fifo_data_out[1];
	assign dout_1_endofpacket = alpha_fifo_data_out[0];
	assign dout_1_data = alpha_fifo_data_out[BITS_PER_SYMBOL_ALPHA+1:2];
	
	generate 
		if (((IS_422 > 0) && (SYMBOLS_PER_BEAT_VIDEO_IN < 3)) || ((IS_422 == 0) && ((SYMBOLS_PER_BEAT_VIDEO_IN == 3) || (SYMBOLS_PER_BEAT_VIDEO_IN == 1)))) begin
			assign allow_flush = !sym_count[SYM_COUNT_LEN-1];
		end else begin
			assign allow_flush = 1'b1;
		end
	endgenerate
	
	generate
		if (ENABLE_COMPONENT_1 > 1) begin
			reg [2:0] comp_shift;
			always @(posedge clock or posedge reset) begin
				if (reset) begin
					comp_shift <= 3'b111;
				end else begin
					if ((valid_str[0] || flush[1]) && ready_reg) begin
						comp_shift[0] <= !comp_1;
					end
					if ((valid_str[0] || flush[2]) && ready_reg) begin
						comp_shift[1] <= comp_shift[0];
					end
					if ((valid_str[0] || flush[3]) && ready_reg) begin
						comp_shift[2] <= comp_shift[1];
					end
				end
			end
			assign comp_1_shift = comp_shift[2];
		end else begin
			assign comp_1_shift = 1'b1;
		end
	endgenerate
	
	generate 
		case (ENABLE_COMPONENT_1)
			default	:	assign comp_1_disable = 1'b1;
			1		:	assign comp_1_disable = 1'b0;
			2		:   assign comp_1_disable = comp_1;
		endcase
	endgenerate
	
	generate
		if (ENABLE_CLEAN_SWITCHING > 0) begin
			reg [RANGE_X2*STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO - 1:0] next_match_value [NUM_COLOURS-1:0];
			reg switch_str;
			reg addr_prefix;
			reg comp_1_2, alpha_replace_2, video_replace_2;
			reg [NUM_COLOURS-1:0] col_enables_2;
			reg [1:0] video_replace_delay;
			reg [1:0] alpha_replace_delay;
			always @ (posedge clock or posedge reset) begin
				if(reset) begin
					switch_str <= 1'b0;
					addr_prefix <= 1'b0;
					alpha_replace_2 <= 1'b0;
					video_replace_2 <= 1'b0;
					comp_1_2 <= 1'b0;
					col_enables_2 <= 0;
					video_replace_delay <= 2'b00;
					alpha_replace_delay <= 2'b00;
				end else begin
					if ((valid_str[0] || flush[0]) && ready_reg) begin
						if (eop_str[0] && is_video[0] && do_switch) begin
							switch_str <= 1'b1;
							for (u=0; u < NUM_COLOURS; u=u+1) begin
								match_value[u] <= next_match_value[u];
							end
							alpha_replace_2 <= alpha_replace_1;
							video_replace_2 <= video_replace_1;
							comp_1_2 <= comp_1_1;
							col_enables_2 <= col_enables_1;
						end else begin
							switch_str <= 1'b0;
						end
					end
					if ((valid_str[0] || flush[1]) && ready_reg) begin
						video_replace_delay[0] <= video_replace_2;
						alpha_replace_delay[0] <= alpha_replace_2;
						if (switch_str && (valid_str[0] || flush[0])) begin
							addr_prefix <= ! addr_prefix;
						end
					end
					if ((valid_str[0] || flush[2]) && ready_reg) begin
						video_replace_delay[1] <= video_replace_delay[0];
						alpha_replace_delay[1] <= alpha_replace_delay[0];
					end
					if (av_chipselect_str && av_write_str && (av_address_str[ADDRESS_WIDTH_2-1:ADDRESS_WIDTH_2-2] == 2'b10)) begin
						for (init_i = 0; init_i < NUM_COLOURS; init_i=init_i+1) begin
							if (init_i == av_address_str[ADDRESS_WIDTH-1:0]) begin
								next_match_value[init_i] <= match_in;
							end
						end
					end
				end
			end
			assign w_address = {(!addr_prefix),av_address_str[ADDRESS_WIDTH-1:0]};
			assign r_address = {addr_prefix,address};
			assign alpha_replace = alpha_replace_delay[1];
			assign video_replace = video_replace_delay[1];
			assign comp_1 = comp_1_2;
			assign col_enables = col_enables_2;
		end else begin
			always @ (posedge clock) begin
				if (av_chipselect_str && av_write_str && (av_address_str[ADDRESS_WIDTH_2-1:ADDRESS_WIDTH_2-2] == 2'b10)) begin
					for (init_i = 0; init_i < NUM_COLOURS; init_i=init_i+1) begin
						if (init_i == av_address_str[ADDRESS_WIDTH-1:0]) begin
							match_value[init_i] <= match_in;
						end
					end
				end
			end
			assign w_address = av_address_str[ADDRESS_WIDTH-1:0];
			assign r_address = address;
			assign alpha_replace = alpha_replace_1;
			assign video_replace = video_replace_1;
			assign comp_1 = comp_1_1;
			assign col_enables = col_enables_1;
		end
	endgenerate
	
	generate
		if (IS_422) begin
			wire [STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO - 1:0] temp_mem_in;
			if (ENABLE_COMPONENT_1 == 0) begin
				if (DISABLE_ALPHA_REPLACE > 0) begin
					assign temp_mem_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin 
					assign temp_mem_in = {av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
				if (ENABLE_RANGE > 0) begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
			end else begin
				if (DISABLE_ALPHA_REPLACE > 0) begin
					assign temp_mem_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin
					assign temp_mem_in = {av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_4[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
				if (ENABLE_RANGE > 0) begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_4[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_5[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_6[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_7[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
			end
			if (DISABLE_ALPHA_REPLACE > 0) begin
				assign mem_data_in = temp_mem_in;
				assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (do_alpha_sop[SHIFT_LENGTH+1]) ? 0 : alpha_fifo_in;
				assign alpha_save_in = 0;
			end else begin
				if (DISABLE_VIDEO_REPLACE > 0) begin
					assign mem_data_in = {av_writedata_str[BITS_PER_SYMBOL_ALPHA-1:0]};
					assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (do_alpha_sop[SHIFT_LENGTH+1]) ? 0 : (do_alpha_replace) ? mem_data_str[BITS_PER_SYMBOL_ALPHA-1:0] : alpha_fifo_in;
				end else begin
					assign mem_data_in = {av_writedata_str[BITS_PER_SYMBOL_ALPHA-1:0],temp_mem_in};
					assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (do_alpha_sop[SHIFT_LENGTH+1]) ? 0 : (do_alpha_replace) ? mem_data_str[STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO+BITS_PER_SYMBOL_ALPHA-1:STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO] : alpha_fifo_in;
				end
				assign alpha_save_in = mem_data_str[LOCAL_MEM_DATA_W-1:LOCAL_MEM_DATA_W-BITS_PER_SYMBOL_ALPHA];
			end
			assign alpha_valid_in = ready_reg && (enable_sig_video || (flush[SHIFT_LENGTH+2] && allow_flush)) && valid_str[SHIFT_LENGTH+2] && ((is_video[SHIFT_LENGTH+2] && (sym_delayed[SHIFT_LENGTH+1] || second_alpha_sig || early_eop[SHIFT_LENGTH+1])) || do_alpha_sop[SHIFT_LENGTH+1]);
		end else begin
			wire [STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO - 1:0] temp_mem_in;
			if (DISABLE_ALPHA_REPLACE > 0) begin
				assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (do_alpha_sop[SHIFT_LENGTH+1]) ? 0 : alpha_fifo_in;
				assign mem_data_in = temp_mem_in;
			end else begin
				if (DISABLE_VIDEO_REPLACE > 0) begin
					assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (do_alpha_sop[SHIFT_LENGTH+1]) ? 0 : (do_alpha_replace) ? mem_data_str[BITS_PER_SYMBOL_ALPHA-1:0] : alpha_fifo_in;
					assign mem_data_in = {av_writedata_str[BITS_PER_SYMBOL_ALPHA-1:0]};
				end else begin
					assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (do_alpha_sop[SHIFT_LENGTH+1]) ? 0 : (do_alpha_replace) ? mem_data_str[STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO+BITS_PER_SYMBOL_ALPHA-1:STR_SYMBOLS*BITS_PER_SYMBOL_VIDEO] : alpha_fifo_in;
					assign mem_data_in = {av_writedata_str[BITS_PER_SYMBOL_ALPHA-1:0],temp_mem_in};
				end
			end
			if (ENABLE_COMPONENT_1 == 0) begin
				if (DISABLE_VIDEO_REPLACE > 0) begin
					assign temp_mem_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin
					assign temp_mem_in = {av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
				if (ENABLE_RANGE > 0) begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
			end else begin
				if (DISABLE_VIDEO_REPLACE > 0) begin
					assign temp_mem_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin
					assign temp_mem_in = {av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
				if (ENABLE_RANGE > 0) begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_3[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_4[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_5[BITS_PER_SYMBOL_VIDEO-1:0]};
				end else begin
					assign match_in = {av_writedata_str[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_1[BITS_PER_SYMBOL_VIDEO-1:0],av_writedata_str_2[BITS_PER_SYMBOL_VIDEO-1:0]};
				end
			end
			assign alpha_valid_in = ready_reg && (enable_sig_video || (flush[SHIFT_LENGTH+2] && allow_flush)) && valid_str[SHIFT_LENGTH+2] && ((is_video[SHIFT_LENGTH+2] && (sym_delayed[SHIFT_LENGTH+1] || early_eop[SHIFT_LENGTH+1])) || do_alpha_sop[SHIFT_LENGTH+1]);
		end 
	endgenerate
		
	generate
		genvar i;
		if (SYMBOLS_PER_BEAT_VIDEO == 2) begin
			if (IS_422 == 0) begin
				if (ENABLE_RANGE > 0) begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_one
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video ||(flush[1] && allow_flush))) begin
									if (((comp_in[i][BITS_PER_SYMBOL_VIDEO-1:0] < data_str[0][BITS_PER_SYMBOL_VIDEO-1:0]) || (sym_count[SHIFT_LENGTH-1] && comp_1_disable)) 
									&& ((comp_in[i][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] < data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]) || sym_count[SHIFT_LENGTH-1])
									&& ((comp_in[i][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] > data_str[0][BITS_PER_SYMBOL_VIDEO-1:0]) || (sym_count[SHIFT_LENGTH-1] && comp_1_disable)) 
									&& ((comp_in[i][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO] > data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]) || sym_count[SHIFT_LENGTH-1])) begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end
						end
					end
				end else begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_one
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video ||(flush[1] && allow_flush))) begin
									if (((comp_in[i][BITS_PER_SYMBOL_VIDEO-1:0] == data_str[0][BITS_PER_SYMBOL_VIDEO-1:0]) || (sym_count[SHIFT_LENGTH-1] && comp_1_disable)) 
									&& ((comp_in[i][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] == data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]) || sym_count[SHIFT_LENGTH-1])) begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end
						end
					end
				end
			end else begin
				if (ENABLE_RANGE > 0) begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_three
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video || (flush[1] && allow_flush))) begin
									if (((comp_in[i][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] < data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]) || comp_1_disable) 
									&& (comp_in[i][BITS_PER_SYMBOL_VIDEO-1:0] < data_str[0][BITS_PER_SYMBOL_VIDEO-1:0])
									&& ((comp_in[i][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO] > data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]) || comp_1_disable) 
									&& (comp_in[i][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] > data_str[0][BITS_PER_SYMBOL_VIDEO-1:0])) begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end
						end
					end
				end else begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_three
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video || (flush[1] && allow_flush))) begin
									if (((comp_in[i][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] == data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]) || comp_1_disable) 
									&& (comp_in[i][BITS_PER_SYMBOL_VIDEO-1:0] == data_str[0][BITS_PER_SYMBOL_VIDEO-1:0])) begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end
						end
					end
				end
			end
		end else begin
			if (SYMBOLS_PER_BEAT_VIDEO == 1) begin
				wire temp_wire;
				if (IS_422 == 1) begin
					assign temp_wire = sym_count[SHIFT_LENGTH] || sym_count[1];
				end else begin
					assign temp_wire = sym_count[SHIFT_LENGTH-1];
				end
				if (ENABLE_RANGE > 0) begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_six
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video || (flush[1] && allow_flush))) begin
									if (((comp_in[i][BITS_PER_SYMBOL_VIDEO-1:0] < data_str[0][BITS_PER_SYMBOL_VIDEO-1:0]) || (temp_wire && comp_1_disable)) 
									&& ((comp_in[i][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] > data_str[0][BITS_PER_SYMBOL_VIDEO-1:0]) || (temp_wire && comp_1_disable)))begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end	
						end
					end
				end else begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_six
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video || (flush[1] && allow_flush))) begin
									if ((comp_in[i] == data_str[0]) || (temp_wire && comp_1_disable)) begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end	
						end
					end
				end
			end else begin
				if (ENABLE_RANGE > 0) begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_three
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video || (flush[1] && allow_flush))) begin
									if (((comp_in[i][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] < data_str[0][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]) || (sym_count[SHIFT_LENGTH-1] && comp_1_disable)) 
									&& (comp_in[i][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] < data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO])
									&& (comp_in[i][BITS_PER_SYMBOL_VIDEO-1:0] < data_str[0][BITS_PER_SYMBOL_VIDEO-1:0])
									&& ((comp_in[i][6*BITS_PER_SYMBOL_VIDEO-1:5*BITS_PER_SYMBOL_VIDEO] > data_str[0][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]) || (sym_count[SHIFT_LENGTH-1] && comp_1_disable)) 
									&& (comp_in[i][5*BITS_PER_SYMBOL_VIDEO-1:4*BITS_PER_SYMBOL_VIDEO] > data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO])
									&& (comp_in[i][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO] > data_str[0][BITS_PER_SYMBOL_VIDEO-1:0])) begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end
						end
					end
				end else begin
					for(i = 0; i < NUM_COLOURS; i=i+1) begin : loop_three
						always @ (posedge clock or posedge reset) begin
							if(reset) begin
								select[i] <= 1'b0;
							end else begin
								if (ready_reg && (enable_sig_video || (flush[1] && allow_flush))) begin
									if (((comp_in[i][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] == data_str[0][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]) || (sym_count[SHIFT_LENGTH-1] && comp_1_disable)) 
									&& (comp_in[i][2*BITS_PER_SYMBOL_VIDEO-1:0] == data_str[0][2*BITS_PER_SYMBOL_VIDEO-1:0])) begin
										select[i] <= col_enables[i] && (select[i] || sym_count[0]);
									end else begin
										select[i] <= 1'b0;
									end
								end
							end
						end
					end
				end
			end
		end
	endgenerate
	
	generate
		genvar j;
		for(j = 0; j < ADDRESS_WIDTH; j=j+1) begin : loop_two
			assign address[j] = |alt_repack(j, select);
		end
	endgenerate
	
	generate
		if (SYM_COUNT_LEN > 1) begin
			integer m;
			always @(posedge clock or posedge reset) begin
				if (reset) begin
					sym_count <= 1;
				end else begin
					if (ready_reg && valid_str[0] && is_video[0]) begin
						for (m = SYM_COUNT_LEN-1; m > 0; m=m-1) begin : loop_one
							if (eop_str[0]) begin
								sym_count[m] <= 1'b0;
							end else begin
								sym_count[m] <= sym_count[m-1];
							end
						end
						if (eop_str[0]) begin
							sym_count[0] <= 1'b1;
						end else begin
							sym_count[0] <= sym_count[SYM_COUNT_LEN-1];
						end
					end
				end
			end
		end else begin
			always @(posedge clock) begin
				sym_count[0] <= 1'b1;
			end
		end
	endgenerate
	
	generate 
		genvar p;
		if (IS_422 == 0) begin
			if (SHIFT_LENGTH == 2) begin
				if (ENABLE_COMPONENT_1 == 0) begin
					for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
						assign comp_in[p] = match_value[p];
					end
				end else begin
					if (ENABLE_RANGE > 0) begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? {match_value[p][5*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO],match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:0]} : 
													{match_value[p][5*BITS_PER_SYMBOL_VIDEO-1:4*BITS_PER_SYMBOL_VIDEO],match_value[p][6*BITS_PER_SYMBOL_VIDEO-1:5*BITS_PER_SYMBOL_VIDEO],match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO],match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]};
						end
					end else begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:0] : 
													{match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO],match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]};
						end
					end
				end
			end else begin
				if (SHIFT_LENGTH == 3) begin
					if (ENABLE_COMPONENT_1 == 0) begin
						if (ENABLE_RANGE > 0) begin
							for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_5
								assign comp_in[p] = (sym_count[0]) ? {match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO],match_value[p][BITS_PER_SYMBOL_VIDEO-1:0]} : 
														{match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO],match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]};
							end
						end else begin
							for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_5
								assign comp_in[p] = (sym_count[0]) ? match_value[p][BITS_PER_SYMBOL_VIDEO-1:0] : 
														match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO];
							end
						end
					end else begin
						if (ENABLE_RANGE > 0) begin
							for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_5
								assign comp_in[p] = (sym_count[0]) ? {match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO],match_value[p][BITS_PER_SYMBOL_VIDEO-1:0]} : 
														(sym_count[1]) ? {match_value[p][5*BITS_PER_SYMBOL_VIDEO-1:4*BITS_PER_SYMBOL_VIDEO],match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]} : 
															{match_value[p][6*BITS_PER_SYMBOL_VIDEO-1:5*BITS_PER_SYMBOL_VIDEO],match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]};
							end
						end else begin
							for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_5
								assign comp_in[p] = (sym_count[0]) ? match_value[p][BITS_PER_SYMBOL_VIDEO-1:0] : 
														(sym_count[1]) ? match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : 
															match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO];
							end
						end
					end
				end else begin
					if (ENABLE_COMPONENT_1 == 0) begin
						if (ENABLE_RANGE > 0) begin
							for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_6
								assign comp_in[p] = {{BITS_PER_SYMBOL_VIDEO{1'b0}},match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO],{BITS_PER_SYMBOL_VIDEO{1'b0}},match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:0]}; 
							end
						end else begin
							for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_6
								assign comp_in[p] = {{BITS_PER_SYMBOL_VIDEO{1'b0}}, match_value[p]}; 
							end
						end		
					end else begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_6
							assign comp_in[p] = match_value[p]; 
						end
					end
				end
			end
		end else begin
			if (SHIFT_LENGTH == 2) begin
				if (ENABLE_COMPONENT_1 == 0) begin
					if (ENABLE_RANGE > 0) begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? {{BITS_PER_SYMBOL_VIDEO{1'b0}}, match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO],{BITS_PER_SYMBOL_VIDEO{1'b0}}, match_value[p][BITS_PER_SYMBOL_VIDEO-1:0]} : 
													{{BITS_PER_SYMBOL_VIDEO{1'b0}}, match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO],{BITS_PER_SYMBOL_VIDEO{1'b0}}, match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]};
						end
					end else begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? {{BITS_PER_SYMBOL_VIDEO{1'b0}}, match_value[p][BITS_PER_SYMBOL_VIDEO-1:0]} : 
													{{BITS_PER_SYMBOL_VIDEO{1'b0}}, match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]};
						end
					end
				end else begin
					if (ENABLE_RANGE > 0) begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? {match_value[p][6*BITS_PER_SYMBOL_VIDEO-1:4*BITS_PER_SYMBOL_VIDEO],match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:0]} : 
													{match_value[p][8*BITS_PER_SYMBOL_VIDEO-1:6*BITS_PER_SYMBOL_VIDEO],match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]};
						end
					end else begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:0] : 
													match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO];
						end
					end
				end
			end else begin
				if (ENABLE_COMPONENT_1 == 0) begin
					if (ENABLE_RANGE > 0) begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? {match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO],match_value[p][BITS_PER_SYMBOL_VIDEO-1:0]} : 
													{match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO],match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]};									
						end
					end else begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? match_value[p][BITS_PER_SYMBOL_VIDEO-1:0] : 
													match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO];									
						end
					end
				end else begin
					if (ENABLE_RANGE > 0) begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? {match_value[p][5*BITS_PER_SYMBOL_VIDEO-1:4*BITS_PER_SYMBOL_VIDEO],match_value[p][BITS_PER_SYMBOL_VIDEO-1:0]} : 
													(sym_count[1]) ? {match_value[p][6*BITS_PER_SYMBOL_VIDEO-1:5*BITS_PER_SYMBOL_VIDEO],match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO]} : 
														(sym_count[3]) ? {match_value[p][7*BITS_PER_SYMBOL_VIDEO-1:6*BITS_PER_SYMBOL_VIDEO],match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO]} :
															{match_value[p][8*BITS_PER_SYMBOL_VIDEO-1:7*BITS_PER_SYMBOL_VIDEO],match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO]};
						end
					end else begin
						for (p = 0; p < NUM_COLOURS; p=p+1) begin : loop_four
							assign comp_in[p] = (sym_count[0]) ? match_value[p][BITS_PER_SYMBOL_VIDEO-1:0] : 
													(sym_count[1]) ? match_value[p][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : 
														(sym_count[3]) ? match_value[p][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] :
															match_value[p][4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO];
						end
					end
				end
			end
		end
	endgenerate
	
	
	generate 
		if (IS_422 == 0) begin
			if (SHIFT_LENGTH == 2) begin
				wire [2*BITS_PER_SYMBOL_VIDEO-1:0] vid_replace;
				always @(posedge clock or posedge reset) begin
					if (reset) begin
						sop_str[4] <= 1'b0;
						eop_str[4] <= 1'b0;
						valid_str[4] <= 1'b0;
						is_video[4] <= 1'b0;
						sym_delayed[3] <= 1'b0;
						do_alpha_sop[3] <= 1'b0;
						early_eop[3] <= 1'b0;
					end else begin
						if (ready_reg && (enable_sig_video || (flush[4] && allow_flush))) begin
							data_str[4] <= vid_replace;
							sop_str[4] <= sop_str[3];
							eop_str[4] <= eop_str[3];
							is_video[4] <= is_video[3];
							sym_delayed[3] <= sym_delayed[2];
							do_alpha_sop[3] <= do_alpha_sop[2];
							early_eop[3] <= early_eop[2];
							if (enable_sig_video || flush[3]) begin
								valid_str[4] <= valid_str[3];
							end else begin
								valid_str[4] <= 1'b0;
							end
						end
					end
				end
				if ((DISABLE_VIDEO_REPLACE > 0) && (DISABLE_ALPHA_REPLACE == 0)) begin
					assign vid_replace[BITS_PER_SYMBOL_VIDEO-1:0] = data_str[3][BITS_PER_SYMBOL_VIDEO-1:0];
					assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO+1:2] = data_str[SHIFT_LENGTH+2];
				end else begin
					if (ENABLE_COMPONENT_1 == 0) begin
						assign vid_replace[BITS_PER_SYMBOL_VIDEO-1:0] = data_str[3][BITS_PER_SYMBOL_VIDEO-1:0];
						assign video_fifo_data_in[2*BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2];
					end else begin
						assign vid_replace[BITS_PER_SYMBOL_VIDEO-1:0] = (do_video_replace && comp_1_shift) ? mem_data_str[3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] : data_str[3][BITS_PER_SYMBOL_VIDEO-1:0];
						assign video_fifo_data_in[2*BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2];
					end
				end
				assign vid_replace[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] = data_str[3][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO];
			end else begin
				if (SHIFT_LENGTH == 3) begin
					wire [BITS_PER_SYMBOL_VIDEO-1:0] vid_replace [1:0];
					always @(posedge clock or posedge reset) begin
						if (reset) begin
							sop_str[4] <= 1'b0;
							eop_str[4] <= 1'b0;
							valid_str[4] <= 1'b0;
							sop_str[5] <= 1'b0;
							eop_str[5] <= 1'b0;
							valid_str[5] <= 1'b0;
							is_video[4] <= 1'b0;
							sym_delayed[3] <= 1'b0;
							do_alpha_sop[3] <= 1'b0;
							early_eop[3] <= 1'b0;
							is_video[5] <= 1'b0;
							sym_delayed[4] <= 1'b0;
							do_alpha_sop[4] <= 1'b0;
							early_eop[4] <= 1'b0;
						end else begin
							if (ready_reg && (enable_sig_video || (flush[4] && allow_flush))) begin
								data_str[4] <= vid_replace[0];
								sop_str[4] <= sop_str[3];
								eop_str[4] <= eop_str[3];
								is_video[4] <= is_video[3];
								sym_delayed[3] <= sym_delayed[2];
								do_alpha_sop[3] <= do_alpha_sop[2];
								early_eop[3] <= early_eop[2];
								if (enable_sig_video || flush[3]) begin
									valid_str[4] <= valid_str[3];
								end else begin
									valid_str[4] <= 1'b0;
								end
							end
							if (ready_reg && (enable_sig_video ||(flush[5] && allow_flush))) begin
								data_str[5] <= vid_replace[1];
								sop_str[5] <= sop_str[4];
								eop_str[5] <= eop_str[4];
								is_video[5] <= is_video[4];
								sym_delayed[4] <= sym_delayed[3];
								do_alpha_sop[4] <= do_alpha_sop[3];
								early_eop[4] <= early_eop[3];
								if (enable_sig_video || flush[4]) begin
									valid_str[5] <= valid_str[4];
								end else begin
									valid_str[5] <= 1'b0;
								end
							end
						end
					end
					if ((DISABLE_VIDEO_REPLACE > 0) && (DISABLE_ALPHA_REPLACE == 0)) begin
						assign vid_replace[0] = data_str[3];
						assign vid_replace[1] = data_str[4];
						assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO+1:2] = data_str[SHIFT_LENGTH+2];
					end else begin
						if (ENABLE_COMPONENT_1 == 0) begin
							assign vid_replace[0] = data_str[3];
							assign vid_replace[1] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : data_str[4];
							assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO+1:2] = (do_video_replace) ? mem_data_str[BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2];
						end else begin
							assign vid_replace[0] = (do_video_replace && comp_1_shift) ? mem_data_str[3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] : data_str[3];
							assign vid_replace[1] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : data_str[4];
							assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO+1:2] = (do_video_replace) ? mem_data_str[BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2];
						end
					end
				end else begin
					if ((DISABLE_VIDEO_REPLACE > 0) && (DISABLE_ALPHA_REPLACE == 0)) begin
						assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO+1:2] = data_str[SHIFT_LENGTH+2];
					end else begin
						if (ENABLE_COMPONENT_1 == 0) begin
							assign video_fifo_data_in[2*BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2][2*BITS_PER_SYMBOL_VIDEO-1:0];
							assign video_fifo_data_in[3*BITS_PER_SYMBOL_VIDEO+1:2*BITS_PER_SYMBOL_VIDEO+2] = data_str[SHIFT_LENGTH+2][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO];
						end else begin
							assign video_fifo_data_in[2*BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2][2*BITS_PER_SYMBOL_VIDEO-1:0];
							assign video_fifo_data_in[3*BITS_PER_SYMBOL_VIDEO+1:2*BITS_PER_SYMBOL_VIDEO+2] = (do_video_replace && comp_1_shift) ? mem_data_str[3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] : data_str[SHIFT_LENGTH+2][3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO];
						end
					end	
				end
			end
		end else begin
			if (SHIFT_LENGTH == 2) begin
				wire [2*BITS_PER_SYMBOL_VIDEO-1:0] vid_replace;
				always @(posedge clock or posedge reset) begin
					if (reset) begin
						sop_str[4] <= 1'b0;
						eop_str[4] <= 1'b0;
						valid_str[4] <= 1'b0;
						is_video[4] <= 1'b0;
						sym_delayed[3] <= 1'b0;
						do_alpha_sop[3] <= 1'b0;
						early_eop[3] <= 1'b0;
					end else begin
						if (ready_reg && (enable_sig_video ||(flush[4] && allow_flush))) begin
							data_str[4] <= vid_replace;
							sop_str[4] <= sop_str[3];
							eop_str[4] <= eop_str[3];
							is_video[4] <= is_video[3];
							sym_delayed[3] <= sym_delayed[2];
							do_alpha_sop[3] <= do_alpha_sop[2];
							early_eop[3] <= early_eop[2];
							if (enable_sig_video ||flush[3]) begin
								valid_str[4] <= valid_str[3];
							end else begin
								valid_str[4] <= 1'b0;
							end
						end
					end
				end
				if ((DISABLE_VIDEO_REPLACE > 0) && (DISABLE_ALPHA_REPLACE == 0)) begin
					assign vid_replace = data_str[3];
					assign video_fifo_data_in[2*BITS_PER_SYMBOL_VIDEO+1:2] = data_str[SHIFT_LENGTH+2];
				end else begin
					if (ENABLE_COMPONENT_1 == 0) begin
						assign vid_replace[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] = data_str[3][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO];
						assign vid_replace[BITS_PER_SYMBOL_VIDEO-1:0] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : data_str[3][BITS_PER_SYMBOL_VIDEO-1:0];
						assign video_fifo_data_in[2*BITS_PER_SYMBOL_VIDEO+1:BITS_PER_SYMBOL_VIDEO+2] = data_str[SHIFT_LENGTH+2][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO];
						assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2][BITS_PER_SYMBOL_VIDEO-1:0];
					end else begin
						assign vid_replace[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] = (do_video_replace && comp_1_shift) ? mem_data_str[4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO] :data_str[3][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO];
						assign vid_replace[BITS_PER_SYMBOL_VIDEO-1:0] = (do_video_replace) ? mem_data_str[3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] : data_str[3][BITS_PER_SYMBOL_VIDEO-1:0];
						assign video_fifo_data_in[2*BITS_PER_SYMBOL_VIDEO+1:BITS_PER_SYMBOL_VIDEO+2] = (do_video_replace && comp_1_shift) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : data_str[SHIFT_LENGTH+2][2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO];
						assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2][BITS_PER_SYMBOL_VIDEO-1:0];
					end
				end
			end else begin
				wire [BITS_PER_SYMBOL_VIDEO-1:0] vid_replace [2:0];
				always @(posedge clock or posedge reset) begin
					if (reset) begin
						sop_str[4] <= 1'b0;
						eop_str[4] <= 1'b0;
						valid_str[4] <= 1'b0;
						sop_str[5] <= 1'b0;
						eop_str[5] <= 1'b0;
						valid_str[5] <= 1'b0;
						sop_str[6] <= 1'b0;
						eop_str[6] <= 1'b0;
						valid_str[6] <= 1'b0;
						is_video[4] <= 1'b0;
						sym_delayed[3] <= 1'b0;
						do_alpha_sop[3] <= 1'b0;
						early_eop[3] <= 1'b0;
						is_video[5] <= 1'b0;
						sym_delayed[4] <= 1'b0;
						do_alpha_sop[4] <= 1'b0;
						early_eop[4] <= 1'b0;
						is_video[6] <= 1'b0;
						sym_delayed[5] <= 1'b0;
						do_alpha_sop[5] <= 1'b0;
						early_eop[5] <= 1'b0;
					end else begin
						if (ready_reg && (enable_sig_video ||(flush[4] && allow_flush))) begin
							data_str[4] <= vid_replace[0];
							sop_str[4] <= sop_str[3];
							eop_str[4] <= eop_str[3];
							is_video[4] <= is_video[3];
							sym_delayed[3] <= sym_delayed[2];
							do_alpha_sop[3] <= do_alpha_sop[2];
							early_eop[3] <= early_eop[2];
							if (enable_sig_video ||flush[3]) begin
								valid_str[4] <= valid_str[3];
							end else begin
								valid_str[4] <= 1'b0;
							end
						end
						if (ready_reg && (enable_sig_video ||(flush[5] && allow_flush))) begin
							data_str[5] <= vid_replace[1];
							sop_str[5] <= sop_str[4];
							eop_str[5] <= eop_str[4];
							is_video[5] <= is_video[4];
							sym_delayed[4] <= sym_delayed[3];
							do_alpha_sop[4] <= do_alpha_sop[3];
							early_eop[4] <= early_eop[3];
							if (enable_sig_video ||flush[4]) begin
								valid_str[5] <= valid_str[4];
							end else begin
								valid_str[5] <= 1'b0;
							end
						end
						if (ready_reg && (enable_sig_video ||(flush[6] && allow_flush))) begin
							data_str[6] <= vid_replace[2];
							sop_str[6] <= sop_str[5];
							eop_str[6] <= eop_str[5];
							is_video[6] <= is_video[5];
							sym_delayed[5] <= sym_delayed[4];
							do_alpha_sop[5] <= do_alpha_sop[4];
							early_eop[5] <= early_eop[4];
							if (enable_sig_video ||flush[5]) begin
								valid_str[6] <= valid_str[5];
							end else begin
								valid_str[6] <= 1'b0;
							end
						end
					end
				end
				if ((DISABLE_VIDEO_REPLACE > 0) && (DISABLE_ALPHA_REPLACE == 0)) begin
					assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO+1:2] = data_str[SHIFT_LENGTH+2];
					assign vid_replace[0] = data_str[3];
					assign vid_replace[1] = data_str[4];
					assign vid_replace[2] = data_str[5];
				end else begin
					if (ENABLE_COMPONENT_1 == 0) begin
						assign vid_replace[0] = data_str[3];
						assign vid_replace[1] = (do_video_replace) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : data_str[4];
						assign vid_replace[2] = data_str[5];
						assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2];
					end else begin
						assign vid_replace[0] = (do_video_replace && comp_1_shift) ? mem_data_str[4*BITS_PER_SYMBOL_VIDEO-1:3*BITS_PER_SYMBOL_VIDEO] : data_str[3];
						assign vid_replace[1] = (do_video_replace) ? mem_data_str[3*BITS_PER_SYMBOL_VIDEO-1:2*BITS_PER_SYMBOL_VIDEO] : data_str[4];
						assign vid_replace[2] = (do_video_replace && comp_1_shift) ? mem_data_str[2*BITS_PER_SYMBOL_VIDEO-1:BITS_PER_SYMBOL_VIDEO] : data_str[5];
						assign video_fifo_data_in[BITS_PER_SYMBOL_VIDEO+1:2] = (do_video_replace) ? mem_data_str[BITS_PER_SYMBOL_VIDEO-1:0] : data_str[SHIFT_LENGTH+2];
					end
				end
			end
		end
	endgenerate
						
	always @(posedge clock or posedge reset) begin
		if (reset) begin
			no_replace <= 1'b0;
			do_alpha_replace <= 1'b0;
			do_video_replace <= 1'b0;
			video_in_ready <= 1'b0;
			running <= 1'b0;
			valid_str[3:0] <= 4'b0000;
			eop_str[3:0] <= 4'b0000;
			sop_str[3:0] <= 4'b0000;
			is_video[3:0] <= 4'b0000;
			sym_delayed[2:0] <= 3'b000;
			sym_delayed_1 <= 2'b00;
			ready_reg <= 1'b0;
			do_alpha_sop[2:0] <= 3'b000;
			early_eop[2:0] <= 3'b000;
			flush <= 0;
		end else begin
			ready_reg <= sink_ready;
			video_in_ready <= running && (alpha_ready || dout_1_ready) && (video_ready || dout_0_ready);
			sink_ready <= (alpha_ready || dout_1_ready) && (video_ready || dout_0_ready);
			if (ready_reg) begin
				flush[0] <= video_in_valid && video_in_eop;
				for (r=1; r<SHIFT_LENGTH+3; r=r+1) begin  
					flush[r] <= flush[r-1] || (video_in_valid && video_in_eop);
				end
			end
			if (sop_str[0] && valid_str[0] && (data_str[0][3:0] == 0)) begin
				running <= master_enable;
			end else begin
				if (master_enable) begin
					running <= 1'b1;
				end
			end
			if (ready_reg) begin 
				valid_str[0] <= video_in_valid;
				eop_str[0] <= video_in_eop;
				sop_str[0] <= video_in_sop;
				data_str[0] <= video_in_data;
			end
			if (ready_reg && (valid_str[0] || flush[0])) begin
				if (eop_str[0] && valid_str[0]) begin
					is_video[0] <= 1'b0;
				end else begin
					if (sop_str[0] && (data_str[0][3:0] == 0) && valid_str[0]) begin
						is_video[0] <= 1'b1;
					end
				end
			end
			if (ready_reg && (valid_str[0] || (flush[1] && allow_flush))) begin
				if (sop_str[0] && (data_str[0][3:0] == 0) && (valid_str[0])) begin
					do_alpha_sop[0] <= 1'b1;
				end else begin
					do_alpha_sop[0] <= 1'b0;
				end
			end
			if (ready_reg && (enable_sig_video || (flush[1] && allow_flush))) begin
				if (enable_sig_video || flush[0]) begin
					valid_str[1] <= valid_str[0];
				end else begin
					valid_str[1] <= 1'b0;
				end
				sop_str[1] <= sop_str[0];
				data_str[1] <= data_str[0];
				is_video[1] <= is_video[0];
				sym_delayed[0] <= sym_count[0];
				sym_delayed_1[0] <= sym_count[LAST_INDEX];//change here
				if (eop_str[0] && is_video[0] && !sym_count[SHIFT_LENGTH-1]) begin
					early_eop[0] <= 1'b1;
				end else begin
					early_eop[0] <= 1'b0;
				end
			end
			if (ready_reg && (valid_str[0] || flush[1])) begin
				eop_str[1] <= eop_str[0];
			end
			if (ready_reg && (enable_sig_video || (flush[2]&& allow_flush))) begin
				if (enable_sig_video || flush[1]) begin
					valid_str[2] <= valid_str[1];
				end else begin
					valid_str[2] <= 1'b0;
				end
				eop_str[2] <= eop_str[1];
				sop_str[2] <= sop_str[1];
				data_str[2] <= data_str[1];
				is_video[2] <= is_video[1];
				do_alpha_sop[1] <= do_alpha_sop[0];
				early_eop[1] <= early_eop[0];
				sym_delayed[1] <= sym_delayed[0];
				sym_delayed_1[1] <= sym_delayed_1[0];
				if (select == 0) begin
					no_replace <= 1'b1;
				end else begin
					no_replace <= 1'b0;
				end
			end
			if (ready_reg && (enable_sig_video ||(flush[3]&& allow_flush))) begin
				if (enable_sig_video||flush[2]) begin
					valid_str[3] <= valid_str[2];
				end else begin
					valid_str[3] <= 1'b0;
				end
				eop_str[3] <= eop_str[2];
				sop_str[3] <= sop_str[2];
				data_str[3] <= data_str[2];
				is_video[3] <= is_video[2];
				do_alpha_sop[2] <= do_alpha_sop[1];
				early_eop[2] <= early_eop[1];
				sym_delayed[2] <= sym_delayed[1];
				mem_data_str <= mem_data_out;
				do_alpha_replace <= !no_replace && alpha_replace && sym_delayed_1[1];
				do_video_replace <= !no_replace && video_replace && is_video[2] && sym_delayed_1[1];
			end
		end	
	end
	
	always @ (posedge clock or posedge reset) begin
		if(reset) begin
			master_enable <= 1'b0;
			alpha_replace_1 <= 1'b0;
			video_replace_1 <= 1'b0;
			do_switch <= 1'b0;
			comp_1_1 <= 1'b0;
			col_enables_1 <= 0;
			av_chipselect_str <= 1'b0;
			av_write_str <= 1'b0;
			av_writedata_str <= 0;
			av_writedata_str_1 <= 0;
			av_writedata_str_2 <= 0;
			av_writedata_str_3 <= 0;
			av_writedata_str_4 <= 0;
			av_writedata_str_5 <= 0;
			av_writedata_str_6 <= 0;
			av_writedata_str_7 <= 0;
			av_address_str <= 0;
		end else begin
			av_chipselect_str <= av_chipselect;
			av_write_str <= av_write;
			av_address_str <= av_address;
			if (av_write && av_chipselect) begin
				av_writedata_str_1 <= av_writedata_str[AV_STR_WIDTH-1:0];
				av_writedata_str_2 <= av_writedata_str_1;
				av_writedata_str_3 <= av_writedata_str_2;
				av_writedata_str_4 <= av_writedata_str_3;
				av_writedata_str_5 <= av_writedata_str_4;
				av_writedata_str_6 <= av_writedata_str_5;
				av_writedata_str_7 <= av_writedata_str_6;
				av_writedata_str <= av_writedata[AV_STR_WIDTH-1:0];
			end
			if (av_chipselect_str && av_write_str && (av_address_str[ADDRESS_WIDTH_2-1:ADDRESS_WIDTH_2-2] == 2'b00)) begin
				case (av_address_str)
					0 :	master_enable <= av_writedata_str[0];    
					3 :	alpha_replace_1 <= av_writedata_str[0];
					4 : video_replace_1 <= av_writedata_str[0];
					5 : comp_1_1 <= av_writedata_str[0];
					6 : do_switch <= av_writedata_str[0];
				endcase	
			end else begin
				if ((valid_str[0] || flush[1]) && ready_reg) begin
					if (eop_str[0] && is_video[0]) begin
						do_switch <= 1'b0;
					end
				end
			end
			if (av_chipselect_str && av_write_str && (av_address_str[ADDRESS_WIDTH_2-1:ADDRESS_WIDTH_2-2] == 2'b11)) begin
				for (init_j = 0; init_j < NUM_COLOURS; init_j=init_j+1) begin
					if (init_j == av_address_str[ADDRESS_WIDTH-1:0]) begin
						col_enables_1[init_j] <= av_writedata_str[0];
					end
				end
			end
			if(av_chipselect && ~av_write && (av_address == 1)) begin
				av_readdata <= {{AV_DATA_WIDTH-1{1'b0}}, running};
			end
		end
	end
		
endmodule
					
			