module alt_vip_control_packet_rearrange

	(	clock,
		reset,
		// Avalon-ST sink interface
		din_0_ready,
		din_0_valid,
		din_0_startofpacket,
		din_0_endofpacket,
		din_0_data, 
		// Avalon-ST source interfaces
		dout_0_ready,
		dout_0_valid,
		dout_0_startofpacket,
		dout_0_endofpacket,
		dout_0_data,
		dout_0_split_symbol
	); 
	
	parameter BITS_PER_SYMBOL = 8;
	parameter SYMBOLS_PER_BEAT_IN = 4;
	
	input		wire clock;
	input		wire reset;
	// Avalon-ST sink interface
	output wire din_0_ready;
	input	wire din_0_valid;
	input	wire din_0_startofpacket;
	input	wire din_0_endofpacket;
	input	wire [SYMBOLS_PER_BEAT_IN*BITS_PER_SYMBOL - 1:0] din_0_data; 
	// Avalon-ST source interfaces
	//video_data
	input	wire dout_0_ready;
	output	reg dout_0_valid;
	output	reg dout_0_startofpacket;
	output	reg dout_0_endofpacket;
	output	reg [(SYMBOLS_PER_BEAT_IN-1)*BITS_PER_SYMBOL - 1:0] dout_0_data;
	output	reg [BITS_PER_SYMBOL - 1:0] dout_0_split_symbol;
	
	generate
		if (SYMBOLS_PER_BEAT_IN > 3) begin
		
			reg [2:0] control_word;
			reg sop_str, eop_str, valid_str, ready_reg, sop_buffer, eop_buffer, valid_buffer;
			reg [4*BITS_PER_SYMBOL - 1:0] data_str;
			reg [2*BITS_PER_SYMBOL - 1:0] data_buffer_2;
			reg [4*BITS_PER_SYMBOL - 1:0] data_buffer;
	
			always@(posedge clock or posedge reset) begin
				if (reset) begin
					control_word <= 3'b000;
					valid_str <= 1'b0;
					valid_buffer <= 1'b0;
					ready_reg <= 1'b0;
					dout_0_valid <= 1'b0;
				end else begin
					ready_reg <= din_0_ready;
					dout_0_valid <= dout_0_ready && valid_buffer;
					dout_0_startofpacket <= sop_buffer;
					dout_0_endofpacket <= eop_buffer;
					dout_0_split_symbol <= data_buffer[4*BITS_PER_SYMBOL-1:3*BITS_PER_SYMBOL];
					case (control_word)
						default	:	dout_0_data <= data_buffer[3*BITS_PER_SYMBOL - 1:0];
						3'b010	:   dout_0_data <= {data_buffer[2*BITS_PER_SYMBOL - 1:BITS_PER_SYMBOL], data_buffer[BITS_PER_SYMBOL - 1:0], data_buffer_2[2*BITS_PER_SYMBOL - 1:BITS_PER_SYMBOL]};
						3'b100	:   dout_0_data <= {data_buffer[BITS_PER_SYMBOL - 1:0], data_buffer_2[2*BITS_PER_SYMBOL - 1:BITS_PER_SYMBOL], data_buffer_2[BITS_PER_SYMBOL - 1:0]};
					endcase
					if (ready_reg || dout_0_ready) begin
						sop_str <= din_0_startofpacket;
						eop_str <= din_0_endofpacket;
						if ((valid_str) || (!dout_0_ready)) begin
							valid_str <= din_0_valid;
						end else begin
							valid_str <= 1'b0;
						end
						data_str <= din_0_data;
					end
					if (dout_0_ready) begin
						if (valid_str) begin
							sop_buffer <= sop_str;
							eop_buffer <= eop_str;
							valid_buffer <= 1'b1;
							data_buffer <= data_str;
						end else begin
							sop_buffer <= din_0_startofpacket;
							eop_buffer <= din_0_endofpacket;
							valid_buffer <= din_0_valid;
							data_buffer <= din_0_data;
						end
						if (valid_buffer) begin
							data_buffer_2 <= data_buffer[4*BITS_PER_SYMBOL - 1:2*BITS_PER_SYMBOL];
							if (sop_buffer && (data_buffer[3:0] == 15)) begin
								control_word[0] <= 1'b1;
							end else begin
								control_word[0] <= 1'b0;
							end
							control_word[1] <= control_word[0];
							control_word[2] <= control_word[1];
						end
					end      
				end
			end
	
	      assign din_0_ready = dout_0_ready;
	
		end else begin
	
			reg [4:0] control_word;
			reg [1:0] sop_str, eop_str, valid_str;
			reg ready_reg, sop_buffer, eop_buffer, valid_buffer, internal_ready;
			reg [3*BITS_PER_SYMBOL - 1:0] data_str [1:0];
			reg [3*BITS_PER_SYMBOL - 1:0] data_buffer_2;
			reg [3*BITS_PER_SYMBOL - 1:0] data_buffer;
	
			always@(posedge clock or posedge reset) begin
				if (reset) begin
					control_word <= 6'b000000;
					valid_str[0] <= 1'b0;
					valid_str[1] <= 1'b0;
					valid_buffer <= 1'b0;
					ready_reg <= 1'b0;
					internal_ready <= 1'b0;
					dout_0_valid <= 1'b0;
				end else begin
					ready_reg <= din_0_ready;
					internal_ready <= dout_0_ready && !(control_word[4] && valid_buffer) && !(control_word[3] && valid_buffer) && !(control_word[2] && valid_str[1]) && !(control_word[1] && valid_str[0]);
					dout_0_valid <= dout_0_ready && (valid_buffer || control_word[3] || control_word[4]);
					dout_0_startofpacket <= sop_buffer && !(control_word[3] || control_word[4]);
					dout_0_endofpacket <= (eop_buffer && !control_word[2]) || control_word[4];
					dout_0_split_symbol <= data_buffer[3*BITS_PER_SYMBOL-1:2*BITS_PER_SYMBOL];
					case (control_word)
						default		:	dout_0_data <= data_buffer[2*BITS_PER_SYMBOL-1:0];
						5'b00010	:   dout_0_data <= {data_buffer[BITS_PER_SYMBOL-1:0], data_buffer_2[3*BITS_PER_SYMBOL-1:2*BITS_PER_SYMBOL]};
						5'b00100	:   dout_0_data <= {data_buffer_2[3*BITS_PER_SYMBOL-1:2*BITS_PER_SYMBOL], data_buffer_2[2*BITS_PER_SYMBOL-1:BITS_PER_SYMBOL]};
						5'b01000	:   dout_0_data <= {data_buffer_2[2*BITS_PER_SYMBOL-1:BITS_PER_SYMBOL], data_buffer_2[BITS_PER_SYMBOL-1:0]};
						5'b10000	:   dout_0_data <= {data_buffer[2*BITS_PER_SYMBOL-1:BITS_PER_SYMBOL], data_buffer_2[3*BITS_PER_SYMBOL-1:2*BITS_PER_SYMBOL]};
					endcase
					if (ready_reg || dout_0_ready) begin
						sop_str[0] <= din_0_startofpacket;
						eop_str[0] <= din_0_endofpacket;
						data_str[0] <= din_0_data;
						if ((valid_str[0]) || ((!dout_0_ready || ((control_word[3] || control_word[4]) && valid_buffer)) && valid_str[1])) begin
							valid_str[0] <= din_0_valid;
						end else begin
							valid_str[0] <= 1'b0;
						end
						if ((dout_0_ready && !((control_word[3] || control_word[4]) && valid_buffer)) || !valid_str[1]) begin
						   if (valid_str[0]) begin
						   	  sop_str[1] <= sop_str[0];
							   eop_str[1] <= eop_str[0];
							   data_str[1] <= data_str[0];
							   valid_str[1] <= 1'b1;
					      end else begin
							   sop_str[1] <= din_0_startofpacket;
							   eop_str[1] <= din_0_endofpacket;
							   data_str[1] <= din_0_data;
							   if ((valid_str[1]) || (!dout_0_ready) || ((control_word[3] || control_word[4]) && valid_buffer) ) begin
								   valid_str[1] <= din_0_valid;
							   end else begin
								   valid_str[1] <= 1'b0;
							   end	
						   end
				      end
					end
					if (dout_0_ready) begin
					   if (!(control_word[3] && valid_buffer) && !(control_word[4] && valid_buffer)) begin
						   if (valid_str[1]) begin
							   sop_buffer <= sop_str[1];
							   eop_buffer <= eop_str[1];
							   valid_buffer <= 1'b1;
							   data_buffer <= data_str[1];
						   end else begin
							   sop_buffer <= din_0_startofpacket;
							   eop_buffer <= din_0_endofpacket;
							   valid_buffer <= din_0_valid;
							   data_buffer <= din_0_data;
						   end
						end
						if (valid_buffer) begin
							data_buffer_2 <= data_buffer;
							if (sop_buffer && (data_buffer[3:0] == 15) && !(control_word[3] || control_word[4])) begin
								control_word[0] <= 1'b1;
							end else begin
								control_word[0] <= 1'b0;
							end
						end
						if (valid_buffer || control_word[3] || control_word[4]) begin
							control_word[1] <= control_word[0];
							control_word[2] <= control_word[1];
							control_word[3] <= control_word[2];
							control_word[4] <= control_word[3];
						end
					end      
				end
			end
			
			assign din_0_ready = internal_ready;
	
		end
	endgenerate
		
	
endmodule
