module alt_vip_splitting_alpha_source_core

	(	clock,
		reset,
		// Avalon-ST sink interface
		din_0_ready,
		din_0_valid,
		din_0_startofpacket,
		din_0_endofpacket,
		din_0_data, 
		// Avalon-ST source interfaces
		//video_data
		dout_0_ready,
		dout_0_valid,
		dout_0_startofpacket,
		dout_0_endofpacket,
		dout_0_data,
		//alpha_data
		dout_1_ready,
		dout_1_valid,
		dout_1_startofpacket,
		dout_1_endofpacket,
		dout_1_data,
		//Avalon MM Slave interface
		av_chipselect,
		av_readdata,
		av_write,
		av_writedata,
		av_address
	); 
	
	function integer get_sym_len;
	input integer is_422;
	input integer s_p_b;
	begin
		if (is_422 > 0) begin
			case (s_p_b)
				default	:	get_sym_len = 2;
				2		:	get_sym_len = 4;
				1		:	get_sym_len = 6;
			endcase
		end else begin
			case (s_p_b)
				default	:	get_sym_len = 1;
				3		:	get_sym_len = 2;
				2		:	get_sym_len = 2;
				1		:	get_sym_len = 4;
			endcase
		end
	end
	endfunction
	
	parameter BITS_PER_SYMBOL_VIDEO = 8;
	parameter SYMBOLS_PER_BEAT_VIDEO_IN = 3;
	parameter IS_422 = 0;
	parameter BITS_PER_SYMBOL_ALPHA = 8;                
	parameter FIFO_SIZE = 3;                        // must be >= 3	
	//local parameters
	localparam SYMBOLS_PER_BEAT_VIDEO = (((IS_422 > 0) && (SYMBOLS_PER_BEAT_VIDEO_IN > 2)) || (SYMBOLS_PER_BEAT_VIDEO_IN > 3)) ? SYMBOLS_PER_BEAT_VIDEO_IN-1 : SYMBOLS_PER_BEAT_VIDEO_IN;
	localparam AV_DATA_WIDTH = 32;
	localparam SYM_COUNT_LEN = get_sym_len(IS_422, SYMBOLS_PER_BEAT_VIDEO_IN);
	localparam INDEX = (SYMBOLS_PER_BEAT_VIDEO_IN == 2) ? 1 : 2;
	
	
	input		wire clock;
	input		wire reset;
	// Avalon-ST sink interface
	output	wire din_0_ready;
	input	wire din_0_valid;
	input	wire din_0_startofpacket;
	input	wire din_0_endofpacket;
	input	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO_IN - 1:0] din_0_data; 
	// Avalon-ST source interfaces
	//video_data
	input	wire dout_0_ready;
	output	wire dout_0_valid;
	output	wire dout_0_startofpacket;
	output	wire dout_0_endofpacket;
	output	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] dout_0_data;
	//alpha_data
	input	wire dout_1_ready;
	output	wire dout_1_valid;
	output	wire dout_1_startofpacket;
	output	wire dout_1_endofpacket;
	output	wire [BITS_PER_SYMBOL_ALPHA - 1:0] dout_1_data;
		
	//Avalon MM Slave interface
	input   wire                              av_chipselect;
	output  reg  [AV_DATA_WIDTH-1:0]          av_readdata;
	input   wire                              av_write;
	input   wire [AV_DATA_WIDTH-1:0]          av_writedata;
	input   wire [1:0]		  				  av_address;
	
	reg master_enable, running;
	reg is_video, ready_reg;
	reg [SYM_COUNT_LEN-1:0] sym_count;
	reg video_in_ready;
	
	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO+1:0] video_fifo_data_in, video_fifo_data_out;
	wire [BITS_PER_SYMBOL_ALPHA+1:0] alpha_fifo_data_in, alpha_fifo_data_out;
	wire alpha_valid_in, video_valid_in, alpha_valid_next, video_valid_next;
	wire alpha_ready, video_ready, video_fifo_valid_out, alpha_fifo_valid_out;
	wire video_in_valid, video_in_sop, video_in_eop;
	wire [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] video_in_data;
	wire [BITS_PER_SYMBOL_VIDEO-1:0] alpha_in_data;
	wire alpha_sop;
	
	alt_fifo_block #(
		.DATA_WIDTH (BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO + 2),
		.SIZE (FIFO_SIZE)
	) fifo_inst_0 (
		.clock  (clock),
		.reset (reset),
		.read (dout_0_ready),
		.write (video_valid_in),
		.write_next (video_valid_next),
		.data_in (video_fifo_data_in),
		.data_out (video_fifo_data_out),
		.valid (video_fifo_valid_out),
		.is_space (video_ready)
	);
	
	alt_fifo_block #(
		.DATA_WIDTH (BITS_PER_SYMBOL_ALPHA + 2),
		.SIZE (FIFO_SIZE)
	) fifo_inst_1 (
		.clock  (clock),
		.reset (reset),
		.read (dout_1_ready),
		.write (alpha_valid_in),
		.write_next (alpha_valid_next),
		.data_in (alpha_fifo_data_in),
		.data_out (alpha_fifo_data_out),
		.valid (alpha_fifo_valid_out),
		.is_space (alpha_ready)
	);
	
	generate
		if ((SYMBOLS_PER_BEAT_VIDEO_IN == 4) || ((SYMBOLS_PER_BEAT_VIDEO_IN == 3) && (IS_422 > 0))) begin
			
			alt_vip_control_packet_rearrange #(
				.BITS_PER_SYMBOL (BITS_PER_SYMBOL_VIDEO),
				.SYMBOLS_PER_BEAT_IN  (SYMBOLS_PER_BEAT_VIDEO_IN)
			) rearrange (	
				.clock  (clock),
				.reset  (reset),
				.din_0_ready  (din_0_ready),
				.din_0_valid  (din_0_valid),
				.din_0_startofpacket  (din_0_startofpacket),
				.din_0_endofpacket  (din_0_endofpacket),
				.din_0_data  (din_0_data), 
				.dout_0_ready  (video_in_ready),
				.dout_0_valid  (video_in_valid),
				.dout_0_startofpacket  (video_in_sop),
				.dout_0_endofpacket  (video_in_eop),
				.dout_0_data  (video_in_data),
				.dout_0_split_symbol  (alpha_in_data)
			);
			
			assign alpha_sop = (video_in_sop && video_in_valid && (video_in_data[3:0] == 0)) ? 1'b1 : 1'b0;
			assign video_fifo_data_in[SYMBOLS_PER_BEAT_VIDEO*BITS_PER_SYMBOL_VIDEO+1:2] = video_in_data;
			assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (alpha_sop) ? 0: alpha_in_data;
			assign video_fifo_data_in[1:0] = {video_in_sop, video_in_eop};
			assign alpha_fifo_data_in[1:0] = {alpha_sop, video_in_eop}; 
			assign video_valid_in = video_in_valid;
			assign video_valid_next = video_in_ready;
			assign alpha_valid_in = video_in_valid && (is_video || alpha_sop);
			assign alpha_valid_next = video_in_ready;
			
		end else begin
		   
			reg video_in_valid_reg, video_in_eop_reg, video_in_sop_reg;
		   reg [BITS_PER_SYMBOL_VIDEO * SYMBOLS_PER_BEAT_VIDEO - 1:0] video_in_data_reg;
			
			assign din_0_ready = video_in_ready;
			assign video_in_valid = video_in_valid_reg;
			assign video_in_sop = video_in_sop_reg;
			assign video_in_eop = video_in_eop_reg;
			assign video_in_data = video_in_data_reg;
			assign alpha_sop = (video_in_sop && video_in_valid && (video_in_data[3:0] == 0)) ? 1'b1 : 1'b0;
			assign alpha_fifo_data_in[1:0] = {alpha_sop, video_in_eop}; 
			assign alpha_fifo_data_in[BITS_PER_SYMBOL_ALPHA+1:2] = (alpha_sop) ? 0: alpha_in_data;
			assign alpha_valid_next = ready_reg;
			
			always@(posedge clock or posedge reset) begin
				if (reset) begin
					video_in_valid_reg <= 1'b0;
				end else begin
				   if (ready_reg) begin
					   video_in_valid_reg <= din_0_valid;
			         video_in_sop_reg <= din_0_startofpacket;
			         video_in_eop_reg <= din_0_endofpacket;
			         video_in_data_reg <= din_0_data;
			      end
				end
			end
		  
			if (IS_422 == 0) begin
				if (SYMBOLS_PER_BEAT_VIDEO == 2) begin
					assign alpha_in_data = video_in_data[BITS_PER_SYMBOL_VIDEO+BITS_PER_SYMBOL_ALPHA-1:BITS_PER_SYMBOL_VIDEO];
					assign video_fifo_data_in[1:0] = {video_in_sop, video_in_eop};
					assign video_fifo_data_in[SYMBOLS_PER_BEAT_VIDEO*BITS_PER_SYMBOL_VIDEO+1:2] = video_in_data;
					assign video_valid_in = video_in_valid && ready_reg;
					assign video_valid_next = video_in_ready;
					assign alpha_valid_in = video_in_valid && ((is_video && sym_count[SYM_COUNT_LEN-1]) || alpha_sop) && ready_reg;
				end else begin
				   reg [SYM_COUNT_LEN-1:0] sym_count_1; 
				   reg is_video_2;
					reg valid_str, eop_str, sop_str;
					reg [SYMBOLS_PER_BEAT_VIDEO*BITS_PER_SYMBOL_VIDEO-1:0] data_str;
					always@(posedge clock or posedge reset) begin
						if (reset) begin
							valid_str <= 1'b0;
							is_video_2 <= 1'b0;
						end else begin
						   if (video_in_valid && ready_reg) begin
					         sym_count_1 <= sym_count;
					         is_video_2 <= is_video;
					      end
							if ((!sym_count[SYM_COUNT_LEN-1] || !is_video) && video_in_valid && ready_reg) begin
								valid_str <= video_in_valid;
								sop_str <= video_in_sop;
								eop_str <= video_in_eop;
								data_str <= video_in_data; 
							end
						end
					end
					assign alpha_in_data = video_in_data[BITS_PER_SYMBOL_ALPHA-1:0];
					assign video_fifo_data_in[1:0] = {sop_str, (eop_str || (video_in_eop && sym_count[SYM_COUNT_LEN-1]))};
					assign video_fifo_data_in[SYMBOLS_PER_BEAT_VIDEO*BITS_PER_SYMBOL_VIDEO+1:2] = data_str;
					assign video_valid_in = video_in_valid && valid_str && (!is_video_2 || !sym_count_1[SYM_COUNT_LEN-1]) && ready_reg;
					assign video_valid_next = video_in_ready;
					assign alpha_valid_in = video_in_valid && ((is_video && sym_count[SYM_COUNT_LEN-1]) || alpha_sop) && ready_reg;
				end
			end else begin
			   reg [SYM_COUNT_LEN-1:0] sym_count_1; 
			   reg is_video_2;
				reg valid_str, eop_str, sop_str;
				reg [SYMBOLS_PER_BEAT_VIDEO*BITS_PER_SYMBOL_VIDEO-1:0] data_str;
				always@(posedge clock or posedge reset) begin
					if (reset) begin
						valid_str <= 1'b0;
						is_video_2 <= 1'b0;
						sym_count_1 <= 0;
					end else begin
					   if (video_in_valid && ready_reg) begin
					      sym_count_1 <= sym_count;
							is_video_2 <= is_video;
					   end
						if (((!sym_count[SYM_COUNT_LEN-1] && !sym_count[INDEX]) || !is_video) && video_in_valid && ready_reg) begin
							valid_str <= video_in_valid;
							sop_str <= video_in_sop;
							eop_str <= video_in_eop;
							data_str <= video_in_data;
						end
					end
				end
				assign alpha_valid_in = video_in_valid && ((is_video && (sym_count[SYM_COUNT_LEN-1] || sym_count[INDEX])) || alpha_sop) && ready_reg;
				assign alpha_in_data = video_in_data[BITS_PER_SYMBOL_ALPHA-1:0];
				assign video_fifo_data_in[1:0] = {sop_str, (eop_str || (video_in_eop && (sym_count[SYM_COUNT_LEN-1] || sym_count[INDEX])))};
				assign video_fifo_data_in[SYMBOLS_PER_BEAT_VIDEO*BITS_PER_SYMBOL_VIDEO+1:2] = data_str;
				assign video_valid_in = video_in_valid && valid_str && (!is_video_2 || (!sym_count_1[SYM_COUNT_LEN-1] && !sym_count_1[INDEX])) && ready_reg;
				assign video_valid_next =  video_in_ready;
			end
		end
	endgenerate
	
	generate
		if (SYM_COUNT_LEN > 1) begin
			integer m;
			always @(posedge clock or posedge reset) begin
				if (reset) begin
					sym_count <= 1;
				end else begin
					if (video_in_valid && is_video && ready_reg) begin
						for (m = SYM_COUNT_LEN-1; m > 0; m=m-1) begin : loop_one
							if (video_in_eop) begin
								sym_count[m] <= 1'b0;
							end else begin
								sym_count[m] <= sym_count[m-1];
							end
						end
						if (video_in_eop) begin
							sym_count[0] <= 1'b1;
						end else begin
							sym_count[0] <= sym_count[SYM_COUNT_LEN-1];
						end
					end
				end
			end
		end else begin
			always @(posedge clock) begin
				sym_count[0] <= 1'b1;
			end
		end
	endgenerate
	
	//assinging outputs to shift register outputs 
	assign dout_0_valid = video_fifo_valid_out;
	assign dout_0_startofpacket = video_fifo_data_out[1];
	assign dout_0_endofpacket = video_fifo_data_out[0];
	assign dout_0_data = video_fifo_data_out[BITS_PER_SYMBOL_VIDEO*SYMBOLS_PER_BEAT_VIDEO+1:2];
	assign dout_1_valid = alpha_fifo_valid_out;
	assign dout_1_startofpacket = alpha_fifo_data_out[1];
	assign dout_1_endofpacket = alpha_fifo_data_out[0];
	assign dout_1_data = alpha_fifo_data_out[BITS_PER_SYMBOL_ALPHA+1:2];
						
	always @(posedge clock or posedge reset) begin
		if (reset) begin
			video_in_ready <= 1'b0;
			running <= 1'b0;
			is_video <= 1'b0;
			ready_reg <= 1'b0;
		end else begin
			video_in_ready <= running && (alpha_ready || dout_1_ready) && (video_ready || dout_0_ready);
			ready_reg <= video_in_ready;
			if (video_in_sop && video_in_valid && (video_in_data[3:0] == 0) && ready_reg) begin
				running <= master_enable;
				is_video <= 1'b1;
			end else begin
				if (master_enable) begin
					running <= 1'b1;
				end
				if (video_in_eop && video_in_valid && ready_reg) begin
					is_video <= 1'b0;
				end
			end
		end	
	end
	
	always @ (posedge clock or posedge reset) begin
		if(reset) begin
			master_enable <= 1'b0;
		end else begin
			if (av_chipselect && av_write && (av_address == 0)) begin
				master_enable <= av_writedata[0];    
			end 
			if(av_chipselect && ~av_write && (av_address == 1)) begin
				av_readdata <= {{AV_DATA_WIDTH-1{1'b0}}, running};
			end
		end
	end
		
endmodule
					
			