//--------------------------------------------------------------------------//
// Title:       Helio_VIP_ref_design.v                                                //
// Rev:         Rev 4                                                       //
// Created:     Sept 21, 2012                                               //
//--------------------------------------------------------------------------//
// Description: Cyclone V SX SoC pinout and IO Standard example design      //  
//--------------------------------------------------------------------------//
//Copyright ï½½ 2012 Altera Corporation. All rights reserved.  Altera products  
//are protected under numerous U.S. and foreign patents, maskwork rights,     
//copyrights and other intellectual property laws.                            
//                                                                            
//This reference design file, and your use thereof, is subject to and         
//governed by the terms and conditions of the applicable Altera Reference     
//Design License Agreement.  By using this reference design file, you         
//indicate your acceptance of such terms and conditions between you and       
//Altera Corporation.  In the event that you do not agree with such terms and 
//conditions, you may not use the reference design file. Please promptly      
//destroy any copies you have made.                                           
//                                                                            
//This reference design file being provided on an "as-is" basis and as an     
//accommodation and therefore all warranties, representations or guarantees   
//of any kind (whether express, implied or statutory) including, without      
//limitation, warranties of merchantability, non-infringement, or fitness for 
//a particular purpose, are specifically disclaimed.  By making this          
//reference design file available, Altera expressly does not recommend,       
//suggest or require that this reference design file be used in combination   
//with any other product not provided by Altera.                              
//                                                                            


module Helio_VIP_ref_design (
//--------------------------------------------------------------------------//
//////////////////// Hard Processor System (HPS) pins ////////////////////////
//--------------------------------------------------------------------------//

//HPS-CLK-INPUT------------------------//2 pins   //--------------------------
    //input          clk_osc1,            //3.3V     //Not for instantiation
    //input          clk_osc2,            //3.3V     //Not for instantiation

//HPS-Reset----------------------------//2 pins  //--------------------------
   //inout          mictor_rstn,         //3.3V    //Cold Reset 
   //input          hps_resetn,          //3.3V    //Warm Reset 

//HPS-User-IO--------------------------//X pins  //--------------------------
   //input  [3:0]   user_dipsw_hps,      //1.5V
   //output [3:0]   user_led_hps,        //3.3V
   //input  [1:0]   user_pb_hps,         //1.5V

//HPS-UART----------------------------//2 pins   //--------------------------
    input          uart_rx,            //3.3V LV  //UART Receive
    output         uart_tx,            //3.3V     //UART Transmit
    inout          conv_hps_usb_n,     //3.3V     //Placed on HPS GPIO
    
//HPS-I2C------------------------------//2 pins  //--------------------------
	inout          i2c_scl_hps,         //3.3V    //HPS I2C Clock output
	inout          i2c_sda_hps,         //3.3V    //HPS I2C Data Input/Output
	
//HPS-SPI-Bus--------------------------//4 pins  //--------------------------
	output         spi_csn,             //3.3V    //Slave Sel 0
	input          spi_miso,            //3.3V    //Master Input
	output         spi_mosi,            //3.3V    //Master Output 
	output         spi_sck,             //3.3V    //Clock Output 

//HPS-QSPI-Flash-----------------------//6 pins  //--------------------------
	output         qspi_clk,            //3.3V    //Clock
	inout [3:0]    qspi_io,             //3.3V    //Data
	output         qspi_ss0,            //3.3V    //Select


//HPS-SD-Card-Flash--------------------//7 pins  //--------------------------
    output         sd_clk,              //3.3V    //
    inout          sd_cmd,              //3.3V    //
    //output         sd_pwren,            //3.3V    //
    inout  [3:0]   sd_dat,              //3.3V    //???????????????  should only be (3:0) ????????
    //inout  [7:0]   sd_dat,              //3.3V    //???????????????  should only be (3:0) ????????
    //input          sd_dummy_clkin,      //3.3V    //???????????????  snot required

//HPS-USB-OTG-------------------------//19 pins  //--------------------------
	input          usb_clk,            //3.3V     //Clock
	inout  [7:0]   usb_data,           //3.3V     //
	input          usb_nxt,            //3.3V     //
	input          usb_dir,            //3.3V     //
	output         usb_stp,            //3.3V     //
    inout          usb_reset,          //3.3V     //Placed on HPS GPIO
    
//HPS-Ethernet----10/100/1000----------//14 pins //--------------------------
	output         enet_hps_gtx_clk,    //3.3V    //Gb Ethernet Clock
    inout          enet_hps_intn,       //3.3V    //Placed on HPS GPIO
	output         enet_hps_mdc,        //3.3V    //MDIO Clock (TR=0)
	inout          enet_hps_mdio,       //3.3V    //MDIO Data (TR=0)
	input		   enet_hps_rx_clk,     //3.3V    //Receive Data
	input		   enet_hps_rx_dv,      //3.3V    //Receive Data Valid / Cont
	input  [3:0]   enet_hps_rxd,        //3.3V    //Receive Data
	output         enet_hps_tx_en,      //3.3V    //Transmit Data Enable / Cont
	output [3:0]   enet_hps_txd,        //3.3V    //Transmit Data
/*	
//HPS-CAN-BUS--------------------------//2 pins  //
   input          can_0_rx,            //3.3V    //HPS only
   output         can_0_tx,            //3.3V    //HPS only 
*/

//HPS-Trace----------------------------//10 pins //--------------------------
	//input          mictor_rstn,         //3.3V    //??????????????? not in Qsys - needed?
    output         trace_clk_mic,       //3.3V    //
    output [7:0]   trace_data,          //3.3V    //

//HPS-DDR3-400Mx40---------------------//78 pins //--------------------------
    output [14:0]  ddr3_hps_a,          //SSTL15  //Address
    output [2:0]   ddr3_hps_ba,         //SSTL15  //Bank Address
    output         ddr3_hps_casn,       //SSTL15  //Column Address Strobe
    output         ddr3_hps_cke,        //SSTL15  //Clock Enable
    output         ddr3_hps_clk_n,      //SSTL15  //Diff Clock - Neg
    output         ddr3_hps_clk_p,      //SSTL15  //Diff Clock - Pos
    output         ddr3_hps_csn,        //SSTL15  //Chip Select
    output [3:0]   ddr3_hps_dm,         //SSTL15  //Data Write Mask
    inout  [31:0]  ddr3_hps_dq,         //SSTL15  //Data Bus
    inout  [3:0]   ddr3_hps_dqs_n,      //SSTL15  //Diff Data Strobe - Neg
    inout  [3:0]   ddr3_hps_dqs_p,      //SSTL15  //Diff Data Strobe - Pos
    output         ddr3_hps_odt,        //SSTL15  //On-Die Termination Enable
    output         ddr3_hps_rasn,       //SSTL15  //Row Address Strobe
    output         ddr3_hps_resetn,     //SSTL15  //Reset
    output         ddr3_hps_wen,        //SSTL15  //Write Enable
    input          ddr3_hps_rzq,        //OCT_rzqin
	//input          ddr3_rup,          //OCT_rup
	//input          ddr3_rdn,          //OCT_rdn


//--------------------------------------------------------------------------//
//////////////////////////////// FPGA Pins ///////////////////////////////////
//--------------------------------------------------------------------------//

	
//FPGA-GPLL-CLK------------------------//X pins
    input          clk_100m_fpga,       //2.5V    //100 MHz (2nd copy to max)
    input          clk_50m_fpga,        //2.5V    //50MHz (2nd copy to max) 
	//input          clk_top1,            //2.5V    //156.25 MHz adjustable
    //input          clk_bot1,            //1.5V    //100 MHz ajustable
    input          clk_enet_fpga_p,     //LVDS    //125 MHz fixed

//FPGA-I2C-------------------------//2 pins //--------------------------
	inout          i2c_scl_fpga,        //2.5V-OD //I2C Clock output
	inout          i2c_sda_fpga,        //2.5V-OD //I2C Data Input/Output

//FPGA-User-IO-------------------------//11 pins //--------------------------
	//input          cpu_resetn,          //2.5V    //NIOS CPU Reset Pushbutton
	input  [3:0]   user_dipsw_fpga,     //
	output [3:0]   user_led_fpga,       //
	input  [2:0]   user_pb_fpga,        //
/*
//FPGA-DDR3-400Mx32--------------------//71 pins //--------------------------
   output [14:0]  ddr3_fpga_a,         //SSTL15  //Address
   output [2:0]   ddr3_fpga_ba,        //SSTL15  //Bank Address
	output         ddr3_fpga_casn,      //SSTL15  //Column Address Strobe
	output         ddr3_fpga_cke,       //SSTL15  //Clock Enable
	output         ddr3_fpga_clk_n,     //SSTL15  //Diff Clock - Neg
	output         ddr3_fpga_clk_p,     //SSTL15  //Diff Clock - Pos
   output         ddr3_fpga_csn,       //SSTL15  //Chip Select
   output [3:0]   ddr3_fpga_dm,        //SSTL15  //Data Write Mask
   inout  [31:0]  ddr3_fpga_dq,        //SSTL15  //Data Bus
   inout  [3:0]   ddr3_fpga_dqs_n,     //SSTL15  //Diff Data Strobe - Neg
   inout  [3:0]   ddr3_fpga_dqs_p,     //SSTL15  //Diff Data Strobe - Pos
   output         ddr3_fpga_odt,       //SSTL15  //On-Die Termination Enable
   input          ddr3_fpga_rasn,      //SSTL15  //Row Address Strobe
   input          ddr3_fpga_resetn,    //SSTL15  //Reset
   input          ddr3_fpga_wen,       //SSTL15  //Write Enable
	input          ddr3_fpga_rzq,       //OCT_rzqin
*/
/*
//FPGA-Ethernet1---10/100--------------//14 pins //--------------------------
	input			   enet1_rx_clk,        //2.5V    //Receive Data Clock
	input	[3:0]	   enet1_rx_d,          //2.5V    //Receive Data
	input			   enet1_rx_dv,         //2.5V    //Receive Data Valid
	input			   enet1_rx_error,      //2.5V    //Receive Data Error
	output         enet1_tx_clk_fb,     //2.5V    //Transmit Clock Feedback
	output [3:0]   enet1_tx_d,          //2.5V    //Transmit Data
	output         enet1_tx_en,         //2.5V    //Transmit Data Enable
	input          enet1_tx_error,      //2.5V    //Transmit Data Error
	output         enet_dual_resetn,    //2.5V    //EtherCat PHY Reset
	input			   enet2_rx_clk,        //2.5V    //Receive Data Clock
	input	[3:0]	   enet2_rx_d,          //2.5V    //Receive Data
	input			   enet2_rx_dv,         //2.5V    //Receive Data Valid
	input			   enet2_rx_error,      //2.5V    //Receive Data Error
	output         enet2_tx_clk_fb,     //2.5V    //Transmit Clock Feedback
	output [3:0]   enet2_tx_d,          //2.5V    //Transmit Data
	output         enet2_tx_en,         //2.5V    //Transmit Data Enable
	input          enet2_tx_error,      //2.5V    //Transmit Data Error
*/

/*
//SDI-XCVR-Video-----------------------//X pins  //--------------------------
   //input          clk_148_p,           //LVDS    //148.5MHz Prog. SDI VCXO
	//input          gxb_rx_l4_p,         //PCML    //SDI Receiver (or SMA)
	//output         gxb_tx_l4_p,         //PCML    //SDI Transmitter (or SMA)
	//148.5M programmable VCXO in refclk section above
	output			sdi_clk148_dn,       //2.5V    //VCXO pump up
	output			sdi_clk148_up,       //2.5V    //VCXO pump down
	input 			sdi_fault,           //2.5V    //SDI Cable Driver Fault
	output			sdi_rsti,            //2.5V    //SDI Cable Driver Reset
	output			sdi_rx_bypass,       //2.5V    //SDI Equalizer Bypass
	output 			sdi_rx_en,           //2.5V    //SDI Equalizer Enable
	output 			sdi_tx_en,           //2.5V    //SDI Cable Driver Enable
	output 			sdi_tx_sd_hdn,       //2.5V    //SDI Cable Driver High-Def 
	//SDI driver has I2C - hook to HPS I2C - not the max where it is currently connected ??????????
*/
	
//HSMC-Port-A--------------------------//107pins //--------------------------
   //input          refclk_ql1_p,        //LVPECL  //HSMA Transceiver Refclk -reqs OCT
   //input [3:0]    hsma_rx_p,           //PCML?   //HSMA Receive Data -reqs OCT
   //output [3:0]   hsma_tx_p,           //PCML?   //HSMA Transmit Data
 //Enable below for CMOS HSMC        
	//inout  [79:0]  hsma_d,            //2.5V    //HSMA CMOS Data Bus
 //Enable below for LVDS HSMC        
//   input          hsma_clk_in0,        //2.5V    //Primary single-ended CLKIN
//   input          hsma_clk_in_p1,      //LVDS    //Primary Source-Sync CLKIN
//   input          hsma_clk_in_p2,      //LVDS    //Primary Source-Sync CLKIN
//   output         hsma_clk_out0,       //2.5V    //Primary single-ended CLKOUT
//   output         hsma_clk_out_p1,     //LVDS    //Primary Source-Sync CLKOUT
//   output         hsma_clk_out_p2,     //LVDS    //Primary Source-Sync CLKOUT
//   inout  [3:0]   hsma_d,              //2.5V    //Dedicated CMOS IO
//   input  [16:0]  hsma_rx_d_p,         //LVDS    //LVDS Sounce-Sync Input
//   output [16:0]  hsma_tx_d_p,         //LVDS    //LVDS Sounce-Sync Output
//   input          hsma_prsntn,         //2.5V    //HSMC Presence Detect Input
//   output         hsma_scl,            //2.5V    //SMBus Clock
//   inout          hsma_sda,            //2.5V    //SMBus Data
	//output         hsma_rx_led,         //2.5V    //User LED - Labeled RX
	//output         hsma_tx_led,         //2.5V    //User LED - Labeled TX

//PCI-Express--------------------------//X pins  //--------------------------
/*
   input          pcie_refclk_p,       //LVDS    //PCIe Refclk -reqs OCT 
   input [3:0]    pcie_rx_p,           //PCML    //PCIe Receive Data-req'sOCTé¬©å¹¢ï½½ï½¢éš´è¶£ï½½ï½¢ï½½ï½»
   input [3:0]    pcie_tx_p,           //PCML    //PCIe Transmit Data
   output         pcie_led_g2,         //1.5V    //NOT NEEDED - NO GEN2 -XXXXXXXXXXXXXXXX
   output         pcie_led_x1,         //2.5V
   output         pcie_led_x4,         //2.5V
   input          pcie_perstn,         //2.5V    //PCIe Reset 
*/	
//USB-BLASTER-II-ACCELLORATOR---------//19 pins  //--------------------------
//	input          usb_b2_clk,         //2.5V     //Blaster Clock
//	inout  [7:0]   usb_b2_data,        //2.5V     //Blaster Data
//	input          usb_b2_empty,       //2.5V     //Blaster FIFO Empty
//	input          usb_b2_full,        //2.5V     //Blaster FIFO Full
//	output         usb_b2_oen,         //2.5V     //Blaster Output Enable
//	output         usb_b2_rdn,         //2.5V     //Blaster Read Enable
//	output         usb_b2_resetn,      //2.5V     //Blaster Reset
//	output         usb_b2_scl,         //2.5V     //Blaster I2C Clock
//	inout          usb_b2_sda,         //2.5V     //Blaster I2C Data
//	output         usb_b2_wrn,         //2.5V     //Blaster Enable

	
//---------------------------------------------------------------------------//
//--------------OTHER-PINS-UTILIZED-OUTSIDE-OF-USER-DESIGN-------------------//
//---------------------------------------------------------------------------//
	
//FPGA-Configuration-------------------//X pins  //--------------------------
   //inout          fpga_conf_done,      //2.5V
   //input [15:0]   fpga_config_d,       //2.5V
	//inout          fpga_cvp_confdone,   //2.5V
	//input          fpga_dclk,           //2.5V
	//inout          fpga_nconfig,        //2.5V
	//inout          fpga_nstatus,        //2.5V
	//inout          fpga_pr_done,        //2.5V
	//output         fpga_pr_error,       //2.5V
	//output         fpga_pr_ready,       //2.5V
	//input          fpga_pr_request,     //2.5V


//
// HSMC Connector
//



//Other Signals-------------------//X pins  //--------------------------
//	output           overtemp,         //2.5V
//	output   [2:0]   test_pad,         //2.5V     
//
// output to LCD Multimedia Board via HSMC
//
//NEEK
//	output           HC_NCLK,
//	output           HC_HD,
//	output           HC_VD,
//	output           HC_DEN,
//	output  [7:0]    HC_LCD_DATA,


//map macnica lcd board video out
	output wire [23:0] vid_data, 
	output wire        vid_de,
	output wire        vid_vs, 
	output wire        vid_hs, 
	output wire        vid_pclk,
	output wire        lcd_enable,
	
//map macnica lcd board hdmi rx in
   input wire [7:0] hdmi_rx_y,
	input wire [7:0] hdmi_rx_cbcr,
	input wire       hdmi_rx_hs,
	input wire       hdmi_rx_vs,
	input wire       hdmi_rx_de,
	input wire       hdmi_rx_pclk,
	
//map macnica lcd board interrupts	
	input wire int_rx,
	input wire int_tx,
	input wire int_tp,

//	output           top_HC_GREST,
//	output           top_HC_VGA_CLOCK,
//	output           top_HC_VGA_SYNC,
//	output           top_HC_ADC_CS_N,
//	output           top_HC_ADC_DCLK,
//	output           top_HC_ETH_RESET_N

// 	
// 	input					CH1_CLK,
// 	input		[7:0]		CH1_D,
// 	input					CH1_AVID,
// 	input					CH1_FLD,
// 	input					CH1_HSYNC,
// 	input					CH1_VSYNC,
// 
// 	input					CH2_CLK,
// 	input		[7:0]		CH2_D,
// 	input					CH2_AVID,
// 	input					CH2_FLD,
// 	input					CH2_HSYNC,
// 	input					CH2_VSYNC,
// 
// 	input					CH3_CLK,
// 	input		[7:0]		CH3_D,
// 	input					CH3_AVID,
// 	input					CH3_FLD,
// 	input					CH3_HSYNC,
// 	input					CH3_VSYNC,
// 
// 	input					CH4_CLK,
// 	input		[7:0]		CH4_D,
// 	input					CH4_AVID,
// 	input					CH4_FLD,
// 	input					CH4_HSYNC,
// 	input					CH4_VSYNC,
// 	
//	output		IDCKp,
//	output		IDCKn,
//	output	[11:0]	OUT_D,
//	output		OUT_HSYNC,
//	output		OUT_VSYNC,
//	output		OUT_DE,
	
//	output		QV_RESETB,
//	output		QV_PDN,
//	input		POUT,
	
	inout		SCL,
	inout		SDA

);
//---------------------------------------------------------------------------//
//------------------------Logic description----------------------------------//
//---------------------------------------------------------------------------//
    wire        cpu_resetn;
    wire        h2f_reset_reset_n;
    wire        h2f_cold_reset_reset_n;
    wire        h2f_warm_reset_req_n;
    wire        h2f_user0_clock;
    wire        h2f_user1_clock;
    wire        h2f_user2_clock;
	 wire			 pb_toggle;

	 assign cpu_resetn = h2f_reset_reset_n;	 
	 
//NEEK
//    wire [23:0]	LCD_DATA;
//    wire	LCD_DATAVALID;
//    wire	LCD_HS;
//    wire	LCD_VS;
    
//    reg         refclk_ql1_p_d;
//    reg         clk_enet_fpga_p_d;    
//    reg         clk_100m_fpga_d;
//    reg         clk_50m_fpga_d;
//    reg [1:0]   h2f_user0_clock_cnt;
    
    
 
   
//hps instance--------------------------
//map pll setup   

//generated clocks	
	wire  clk_qsys_cvo_disp;
	wire  clk_qsys_cvo_hdmi_tx;
	wire  clk_qsys_sysclk;
	wire  clk_qsys_ddrclk;
	wire  clk_qsys_cvi_hdmi_rx;

//plls to generate clocks
//system clocks	
	system_pll u_system_pll (
		.refclk   (clk_50m_fpga),          //50mhz input clock
		.rst      (~cpu_resetn),           //reset
		.outclk_0 (clk_qsys_sysclk),       //qsys sys clock
		.outclk_1 (clk_qsys_ddrclk),       //qsys ddr clock
		.locked   ()                       //locked
	);

//hdmi rx clock	
	hdmi_rx_pll u_hdmi_rx_pll (
		.refclk   (hdmi_rx_pclk),     //hdmi rx input clock
		.rst      (~cpu_resetn),          //reset
		.outclk_0 (clk_qsys_cvi_hdmi_rx), //hdmi rx capture clock
		.locked   ()                      //locked
	);

//disp clock	
	disp_pll u_disp_pll (
		.refclk   (clk_50m_fpga),      //50mhz input clock
		.rst      (~cpu_resetn),       //reset
		.outclk_0 (clk_qsys_cvo_disp), //disp cvo clock
		.locked   ()                   //locked
	);

//hdmi tx clock	
	hdmi_tx_pll u_hdmi_tx_pll (
		.refclk   (clk_50m_fpga),         //50mhz input clock
		.rst      (~cpu_resetn),          //reset
		.outclk_0 (clk_qsys_cvo_hdmi_tx), //hdmi tx cvo clock
//		.outclk_1 (hdmi_tx_pclk),
		.locked   ()                      //locked
	);
	
	

//data busses
//display output bus	
	wire [23:0] disp_data;
	wire        disp_de;
	wire        disp_vs;
	wire		   disp_hs;
	wire        disp_pclk;
	
//hdmi tx output bus
	wire [15:0] hdmi_tx_data;
	wire        hdmi_tx_de;
	wire        hdmi_tx_vs;
	wire        hdmi_tx_hs;
	wire        hdmi_tx_pclk;

//hdmi rx bus	
	wire [15:0] hdmi_rx_data;	
	assign hdmi_rx_data [15:8] = hdmi_rx_y;
	assign hdmi_rx_data [7:0]  = hdmi_rx_cbcr;

//disp and hdmi tx pixel clocks	
	assign disp_pclk = clk_qsys_cvo_disp;
	assign hdmi_tx_pclk = clk_qsys_cvo_hdmi_tx;

//change leds to output lcd enable on led0
   wire [3:0]  hps_led;

	assign user_led_fpga[3:1] = hps_led [3:1];
//	assign user_led_fpga[3:1] = ~external_int[2:0];
	assign user_led_fpga[0]   = ~lcd_enable;
	
	
//interrupts
   wire [2:0] external_int;
   assign external_int[2] = int_rx;
	assign external_int[1] = int_tx;
	assign external_int[0] = int_tp;
	
	
	top_hps u_top_hps (
		.sys_clk_clk            (clk_qsys_sysclk),
		.reset_reset_n      	   (cpu_resetn),
		.qsys_ddr_clk_clk       (clk_qsys_ddrclk),
      .qsys_ddr_reset_reset_n (cpu_resetn), 
		// Video Input CH-1
//		.cvi_0_vid_clk		(CH1_CLK),	//  cvi_0.vid_clk
//		.cvi_0_vid_data		(CH1_D),	//  .vid_data
//		.cvi_0_overflow		(),		//  .overflow
//		.cvi_0_vid_datavalid	(CH1_AVID),	//  .vid_datavalid
//		.cvi_0_vid_locked	(1'b1),		//  .vid_locked
//		.cvi_0_vid_v_sync	(CH1_VSYNC),      //  .vid_v_sync
//		.cvi_0_vid_h_sync	(CH1_HSYNC),      //  .vid_h_sync
//		.cvi_0_vid_f		(CH1_FLD),        //  .vid_f
//		// Video Input CH-2
//		.cvi_1_vid_clk		(CH2_CLK),        //  cvi_1.vid_clk
//		.cvi_1_vid_data		(CH2_D),          //  .vid_data
//		.cvi_1_overflow		(),               //  .overflow
//		.cvi_1_vid_datavalid	(CH2_AVID),       //  .vid_datavalid
//		.cvi_1_vid_locked	(1'b1),           //  .vid_locked
//		// Video Input CH-3
//		.cvi_2_vid_clk		(CH3_CLK),        //  cvi_2.vid_clk
//		.cvi_2_vid_data		(CH3_D),          //  .vid_data
//		.cvi_2_overflow		(),               //  .overflow
//		.cvi_2_vid_datavalid	(CH3_AVID),       //  .vid_datavalid
//		.cvi_2_vid_locked	(1'b1),           //  .vid_locked
//		// Video Input CH-4
//		.cvi_3_vid_clk		(CH4_CLK),        //  cvi_3.vid_clk
//		.cvi_3_vid_data		(CH4_D),          //  .vid_data
//		.cvi_3_overflow		(),               //  .overflow
//		.cvi_3_vid_datavalid	(CH4_AVID),       //  .vid_datavalid
//		.cvi_3_vid_locked	(1'b1),           //  .vid_locked
//
		// clocked Video Output using MAI custom component
//		.vco_vid_clk		(clk_33MHz), //  vco.vid_clk
//		.vco_vid_data		(vid_data),   	//    .vid_data
//		.vco_underflow		(),        	//    .underflow
//		.vco_vid_datavalid	(vid_datavalid), //    .vid_datavalid
//		.vco_vid_v_sync		(vid_v_sync),        //  .vid_v_sync
//		.vco_vid_h_sync		(vid_h_sync),        //  .vid_h_sync
//		.vco_vid_f		(),        	//    .vid_f
//		.vco_vid_h		(),        	//    .vid_h
//		.vco_vid_v		(),        	//    .vid_v


//clocked video output using VIP suite and Qsys heirarchy:
//disp connections mp

		.cvo_disp_vid_clk		       (disp_pclk),
		.cvo_disp_vid_data		    (disp_data),
		.cvo_disp_underflow		    (),
      .cvo_disp_vid_mode_change   (),
      .cvo_disp_vid_std           (),
		.cvo_disp_vid_datavalid	    (disp_de),
		.cvo_disp_vid_v_sync		    (disp_vs),
		.cvo_disp_vid_h_sync		    (disp_hs),
		.cvo_disp_vid_f		       (),
		.cvo_disp_vid_h		       (),
		.cvo_disp_vid_v		       (),

//hdmi tx connctions mp		
		.cvo_hdmi_tx_vid_clk        (clk_qsys_cvo_hdmi_tx),
      .cvo_hdmi_tx_vid_data       (hdmi_tx_data),
      .cvo_hdmi_tx_underflow      (),
      .cvo_hdmi_tx_vid_datavalid  (hdmi_tx_de),
      .cvo_hdmi_tx_vid_v_sync     (hdmi_tx_vs),
      .cvo_hdmi_tx_vid_h_sync     (hdmi_tx_hs),
      .cvo_hdmi_tx_vid_f          (),
      .cvo_hdmi_tx_vid_h          (),
      .cvo_hdmi_tx_vid_v          (),
		
		.cvi_hdmi_rx_vid_clk        (clk_qsys_cvi_hdmi_rx),
      .cvi_hdmi_rx_vid_data       (hdmi_rx_data),
      .cvi_hdmi_rx_overflow       (),
      .cvi_hdmi_rx_vid_datavalid  (hdmi_rx_de),
      .cvi_hdmi_rx_vid_locked     (1),
      .cvi_hdmi_rx_vid_v_sync     (hdmi_rx_vs),
      .cvi_hdmi_rx_vid_h_sync     (hdmi_rx_hs),
      .cvi_hdmi_rx_vid_f          (),

      .external_irq_irq_n         (external_int),
		
//		.i2c_sda_pad_io		(SDA),
//		.i2c_scl_pad_io		(SCL),
		
//		.qv_resetb_export	(QV_RESETB),
		
		.led_pio_external_connection_export     (hps_led),     // led_pio_external_connection.export
		.dipsw_pio_external_connection_export   (user_dipsw_fpga),   // dipsw_pio_external_connection.export
		.button_pio_external_connection_export  ({1'b1, user_pb_fpga[1:0]}),      // button_pio_external_connection.export

		.memory_mem_a                    (ddr3_hps_a),   // memory.mem_a
		.memory_mem_ba                   (ddr3_hps_ba),   //       .mem_ba
		.memory_mem_ck                   (ddr3_hps_clk_p),   //       .mem_ck
		.memory_mem_ck_n                 (ddr3_hps_clk_n),   //       .mem_ck_n
		.memory_mem_cke                  (ddr3_hps_cke),   //       .mem_cke
		.memory_mem_cs_n                 (ddr3_hps_csn),   //       .mem_cs_n
		.memory_mem_ras_n                (ddr3_hps_rasn),   //       .mem_ras_n
		.memory_mem_cas_n                (ddr3_hps_casn),   //       .mem_cas_n
		.memory_mem_we_n                 (ddr3_hps_wen),    //       .mem_we_n
		.memory_mem_reset_n              (ddr3_hps_resetn),   //       .mem_reset_n
		.memory_mem_dq                   (ddr3_hps_dq),   //       .mem_dq
		.memory_mem_dqs                  (ddr3_hps_dqs_p),   //       .mem_dqs
		.memory_mem_dqs_n                (ddr3_hps_dqs_n),   //       .mem_dqs_n
		.memory_mem_odt                  (ddr3_hps_odt),   //       .mem_odt
		.memory_mem_dm                   (ddr3_hps_dm),   //       .mem_dm
		.memory_oct_rzqin                (ddr3_hps_rzq),   //       .oct_rzqin
		.hps_io_hps_io_uart0_inst_RX     (uart_rx),   // hps_io.hps_io_uart0_inst_RX
		.hps_io_hps_io_uart0_inst_TX     (uart_tx),   //       .hps_io_uart0_inst_TX
		.hps_io_hps_io_sdio_inst_CMD     (sd_cmd),   //       .hps_io_sdio_inst_CMD
        //.hps_io_hps_io_sdio_inst_PWREN   (sd_pwren),   //       .hps_io_sdio_inst_PWREN
		.hps_io_hps_io_sdio_inst_D0      (sd_dat[0]),   //       .hps_io_sdio_inst_D0
		.hps_io_hps_io_sdio_inst_D1      (sd_dat[1]),   //       .hps_io_sdio_inst_D1
        //.hps_io_hps_io_sdio_inst_D4      (sd_dat[4]),   //       .hps_io_sdio_inst_D4
        //.hps_io_hps_io_sdio_inst_D5      (sd_dat[5]),   //       .hps_io_sdio_inst_D5
        //.hps_io_hps_io_sdio_inst_D6      (sd_dat[6]),   //       .hps_io_sdio_inst_D6
        //.hps_io_hps_io_sdio_inst_D7      (sd_dat[7]),   //       .hps_io_sdio_inst_D7
        //.hps_io_hps_io_sdio_inst_CLK_IN  (sd_dummy_clkin),   //       .hps_io_sdio_inst_CLK_IN
		.hps_io_hps_io_sdio_inst_CLK     (sd_clk),   //       .hps_io_sdio_inst_CLK
		.hps_io_hps_io_sdio_inst_D2      (sd_dat[2]),   //       .hps_io_sdio_inst_D2
		.hps_io_hps_io_sdio_inst_D3      (sd_dat[3]),   //       .hps_io_sdio_inst_D3
		.hps_io_hps_io_i2c0_inst_SDA     (i2c_sda_hps),   //       .hps_io_i2c0_inst_SDA
		.hps_io_hps_io_i2c0_inst_SCL     (i2c_scl_hps),   //       .hps_io_i2c0_inst_SCL
		.hps_io_hps_io_trace_inst_CLK    (trace_clk_mic),   //       .hps_io_trace_inst_CLK
		.hps_io_hps_io_trace_inst_D0     (trace_data[0]),   //       .hps_io_trace_inst_D0
		.hps_io_hps_io_trace_inst_D1     (trace_data[1]),   //       .hps_io_trace_inst_D1
 		.hps_io_hps_io_trace_inst_D2     (trace_data[2]),   //       .hps_io_trace_inst_D2
		.hps_io_hps_io_trace_inst_D3     (trace_data[3]),   //       .hps_io_trace_inst_D3
		.hps_io_hps_io_trace_inst_D4     (trace_data[4]),   //       .hps_io_trace_inst_D4
		.hps_io_hps_io_trace_inst_D5     (trace_data[5]),   //       .hps_io_trace_inst_D5
		.hps_io_hps_io_trace_inst_D6     (trace_data[6]),   //       .hps_io_trace_inst_D6
		.hps_io_hps_io_trace_inst_D7     (trace_data[7]),   //       .hps_io_trace_inst_D7
		.hps_io_hps_io_can0_inst_RX      (),   //       .hps_io_can0_inst_RX
		.hps_io_hps_io_can0_inst_TX      (),   //       .hps_io_can0_inst_TX
		.hps_io_hps_io_qspi_inst_IO0     (qspi_io[0]),   //       .hps_io_qspi_inst_IO0
		.hps_io_hps_io_qspi_inst_IO1     (qspi_io[1]),   //       .hps_io_qspi_inst_IO1
		.hps_io_hps_io_qspi_inst_IO2     (qspi_io[2]),   //       .hps_io_qspi_inst_IO2
		.hps_io_hps_io_qspi_inst_IO3     (qspi_io[3]),   //       .hps_io_qspi_inst_IO3
		.hps_io_hps_io_qspi_inst_SS0     (qspi_ss0),   //       .hps_io_qspi_inst_SS0
		.hps_io_hps_io_qspi_inst_CLK     (qspi_clk),   //       .hps_io_qspi_inst_CLK
		.hps_io_hps_io_spim0_inst_CLK    (spi_sck),     //       .hps_io_spim0_inst_CLK
		.hps_io_hps_io_spim0_inst_MOSI   (spi_mosi),    //       .hps_io_spim0_inst_MOSI
		.hps_io_hps_io_spim0_inst_MISO   (spi_miso),    //       .hps_io_spim0_inst_MISO
		.hps_io_hps_io_spim0_inst_SS0    (spi_csn),     //       .hps_io_spim0_inst_SS0
		.hps_io_hps_io_emac1_inst_TX_CLK (enet_hps_gtx_clk),   //       .hps_io_emac1_inst_TX_CLK
		.hps_io_hps_io_emac1_inst_TXD0   (enet_hps_txd[0]),   //       .hps_io_emac1_inst_TXD0
		.hps_io_hps_io_emac1_inst_TXD1   (enet_hps_txd[1]),   //       .hps_io_emac1_inst_TXD1
		.hps_io_hps_io_emac1_inst_TXD2   (enet_hps_txd[2]),   //       .hps_io_emac1_inst_TXD2
		.hps_io_hps_io_emac1_inst_TXD3   (enet_hps_txd[3]),   //       .hps_io_emac1_inst_TXD3
		.hps_io_hps_io_emac1_inst_RXD0   (enet_hps_rxd[0]),   //       .hps_io_emac1_inst_RXD0
		.hps_io_hps_io_emac1_inst_MDIO   (enet_hps_mdio),   //       .hps_io_emac1_inst_MDIO
		.hps_io_hps_io_emac1_inst_MDC    (enet_hps_mdc),    //       .hps_io_emac1_inst_MDC
		.hps_io_hps_io_emac1_inst_RX_CTL (enet_hps_rx_dv),   //       .hps_io_emac1_inst_RX_CTL
		.hps_io_hps_io_emac1_inst_TX_CTL (enet_hps_tx_en),   //       .hps_io_emac1_inst_TX_CTL
		.hps_io_hps_io_emac1_inst_RX_CLK (enet_hps_rx_clk),   //       .hps_io_emac1_inst_RX_CLK
		.hps_io_hps_io_emac1_inst_RXD1   (enet_hps_rxd[1]),   //       .hps_io_emac1_inst_RXD1
		.hps_io_hps_io_emac1_inst_RXD2   (enet_hps_rxd[2]),   //       .hps_io_emac1_inst_RXD2
		.hps_io_hps_io_emac1_inst_RXD3   (enet_hps_rxd[3]),   //       .hps_io_emac1_inst_RXD3
		.hps_io_hps_io_usb1_inst_D0      (usb_data[0]),   //       .hps_io_usb1_inst_D0
		.hps_io_hps_io_usb1_inst_D1      (usb_data[1]),   //       .hps_io_usb1_inst_D1
		.hps_io_hps_io_usb1_inst_D2      (usb_data[2]),   //       .hps_io_usb1_inst_D2
		.hps_io_hps_io_usb1_inst_D3      (usb_data[3]),   //       .hps_io_usb1_inst_D3
		.hps_io_hps_io_usb1_inst_D4      (usb_data[4]),   //       .hps_io_usb1_inst_D4
		.hps_io_hps_io_usb1_inst_D5      (usb_data[5]),   //       .hps_io_usb1_inst_D5
		.hps_io_hps_io_usb1_inst_D6      (usb_data[6]),   //       .hps_io_usb1_inst_D6
		.hps_io_hps_io_usb1_inst_D7      (usb_data[7]),   //       .hps_io_usb1_inst_D7
		.hps_io_hps_io_usb1_inst_CLK     (usb_clk),   //       .hps_io_usb1_inst_CLK
		.hps_io_hps_io_usb1_inst_STP     (usb_stp),     //       .hps_io_usb1_inst_STP
		.hps_io_hps_io_usb1_inst_DIR     (usb_dir),     //       .hps_io_usb1_inst_DIR
		.hps_io_hps_io_usb1_inst_NXT     (usb_nxt),      //       .hps_io_usb1_inst_NXT
		.hps_io_hps_io_gpio_inst_GPIO00  (usb_reset),  //       .hps_io_gpio_inst_GPIO00
		.hps_io_hps_io_gpio_inst_GPIO09  (conv_hps_usb_n),  //       .hps_io_gpio_inst_GPIO09
		.hps_io_hps_io_gpio_inst_GPIO35  (enet_hps_intn),    //       .hps_io_gpio_inst_GPIO35
		.hps_0_h2f_reset_reset_n                              (h2f_reset_reset_n),      // hps_0_h2f_reset.reset_n
		.hps_0_h2f_cold_reset_reset_n                         (h2f_cold_reset_reset_n), // hps_0_h2f_cold_reset.reset_n
		.hps_0_h2f_warm_reset_handshake_h2f_pending_rst_req_n (h2f_warm_reset_req_n), // hps_0_h2f_warm_reset_handshake.h2f_pending_rst_req_n
		.hps_0_h2f_warm_reset_handshake_f2h_pending_rst_ack_n (h2f_warm_reset_req_n),                    //                               .f2h_pending_rst_ack_n
		.hps_0_h2f_user0_clock_clk                            (h2f_user0_clock),    //          hps_0_h2f_user0_clock.clk
		.hps_0_h2f_user1_clock_clk                            (h2f_user1_clock),    //          hps_0_h2f_user1_clock.clk
		.hps_0_h2f_user2_clock_clk                            (h2f_user2_clock),     //          hps_0_h2f_user2_clock.clk
    .macnica_i2c_ver_0_i2c_scl_pad_io                     (SCL),             //          macnica_i2c_ver_0_i2c.scl_pad_io
    .macnica_i2c_ver_0_i2c_sda_pad_io                     (SDA)              //          .sda_pad_io	
	);
	
//NEEK:
//
// convert 24 bit RGB parallel to 3 x 8 bit parallel for the LCD TDM
// Controller on the NEEK Multimedia module.
//
/*  vga_serial u_lcd_serial( 
    .data	(LCD_DATA),
    .blank	(LCD_DATAVALID),
    .hs		(LCD_HS),
    .vs		(LCD_VS),
    .clk3	(clk_100MHz),
    .data3	(HC_LCD_DATA),
    .blank3	(HC_DEN),
    .hs3	(HC_HD),
    .vs3	(HC_VD)
  );
*/    
// assignments to set NEEK control signals properly (mostly in reset)
//  assign HC_NCLK = clk_100MHz; //k_100MHz_90deg;
//  assign top_HC_GREST = 1'b1;
//  assign top_HC_VGA_CLOCK = clk_100MHz_90deg;
//  assign top_HC_VGA_SYNC = 1'b1;
//  assign top_HC_ADC_CS_N = 1'bz;
//  assign top_HC_ADC_DCLK = 1'b0;
//  assign top_HC_ETH_RESET_N = 1'b0;

//assignments to support the Helio LCD board:

//TJ
//assign lcd_enable = 1'b1;
//lcd_enable controlled via FPGA push button switch SW13 on FPGA board and out to LCD CPLD via HSMA_D2
debounce u_debounce
(
	.reset (cpu_resetn),
	.clk   (clk_50m_fpga),
	.noisy (user_pb_fpga[2]), //stole [2] from QSYS GPIO
	.clean (lcd_enable /*pb_toggle*/)
);	

//assign lcd_enable = ~pb_toggle;

	
//mp switch between disp and hdmi based on button press on the lcd board	
	assign vid_data[23:16] = lcd_enable ? disp_data[23:16] : hdmi_tx_data[7:0];
	assign vid_data[15:8]  = lcd_enable ? disp_data[15:8]  : hdmi_tx_data[15:8];
	assign vid_data[7:0]   = lcd_enable ? disp_data[7:0]   : 8'h00;
	assign vid_hs          = lcd_enable ? disp_hs          : hdmi_tx_hs;
	assign vid_vs          = lcd_enable ? disp_vs          : hdmi_tx_vs;
	assign vid_de          = lcd_enable ? disp_de          : hdmi_tx_de;
	assign vid_pclk        = lcd_enable ? disp_pclk        : hdmi_tx_pclk;	


	
endmodule
